@@ -1,14 +1,15 @@
 # sqlalchemy/exc.py
-# Copyright (C) 2005-2013 the SQLAlchemy authors and contributors <see AUTHORS file>
+# Copyright (C) 2005-2014 the SQLAlchemy authors and contributors
+# <see AUTHORS file>
 #
 # This module is part of SQLAlchemy and is released under
 # the MIT License: http://www.opensource.org/licenses/mit-license.php
 
 """Exceptions used with SQLAlchemy.
 
-The base exception class is :class:`.SQLAlchemyError`.  Exceptions which are
+The base exception class is :exc:`.SQLAlchemyError`.  Exceptions which are
 raised as a result of DBAPI exceptions are all subclasses of
-:class:`.DBAPIError`.
+:exc:`.DBAPIError`.
 
 """
 
@@ -27,6 +28,11 @@ class ArgumentError(SQLAlchemyError):
     """
 
 
+class NoSuchModuleError(ArgumentError):
+    """Raised when a dynamically-loaded module (usually a database dialect)
+    of a particular name cannot be located."""
+
+
 class NoForeignKeysError(ArgumentError):
     """Raised when no foreign keys can be located between two selectables
     during a join."""
@@ -66,12 +72,13 @@ class CircularDependencyError(SQLAlchemyError):
 
     def __reduce__(self):
         return self.__class__, (None, self.cycles,
-                            self.edges, self.args[0])
+                                self.edges, self.args[0])
 
 
 class CompileError(SQLAlchemyError):
     """Raised when an error occurs during SQL compilation"""
 
+
 class UnsupportedCompilationError(CompileError):
     """Raised when an operation is not supported by the given compiler.
 
@@ -82,8 +89,9 @@ class UnsupportedCompilationError(CompileError):
 
     def __init__(self, compiler, element_type):
         super(UnsupportedCompilationError, self).__init__(
-                    "Compiler %r can't render element of type %s" %
-                                (compiler, element_type))
+            "Compiler %r can't render element of type %s" %
+            (compiler, element_type))
+
 
 class IdentifierError(SQLAlchemyError):
     """Raised when a schema name is beyond the max character limit"""
@@ -156,7 +164,7 @@ class NoReferencedColumnError(NoReferenceError):
 
     def __reduce__(self):
         return self.__class__, (self.args[0], self.table_name,
-                            self.column_name)
+                                self.column_name)
 
 
 class NoSuchTableError(InvalidRequestError):
@@ -169,7 +177,7 @@ class UnboundExecutionError(InvalidRequestError):
 
 class DontWrapMixin(object):
     """A mixin class which, when applied to a user-defined Exception class,
-    will not be wrapped inside of :class:`.StatementError` if the error is
+    will not be wrapped inside of :exc:`.StatementError` if the error is
     emitted within the process of executing a statement.
 
     E.g.::
@@ -187,10 +195,6 @@ class DontWrapMixin(object):
                     raise MyCustomException("invalid!")
 
     """
-import sys
-if sys.version_info < (2, 5):
-    class DontWrapMixin:
-        pass
 
 # Moved to orm.exc; compatibility definition installed by orm import until 0.6
 UnmappedColumnError = None
@@ -223,6 +227,10 @@ class StatementError(SQLAlchemyError):
         self.statement = statement
         self.params = params
         self.orig = orig
+        self.detail = []
+
+    def add_detail(self, msg):
+        self.detail.append(msg)
 
     def __reduce__(self):
         return self.__class__, (self.args[0], self.statement,
@@ -231,8 +239,13 @@ class StatementError(SQLAlchemyError):
     def __str__(self):
         from sqlalchemy.sql import util
         params_repr = util._repr_params(self.params, 10)
-        return ' '.join((SQLAlchemyError.__str__(self),
-                         repr(self.statement), repr(params_repr)))
+
+        return ' '.join([
+            "(%s)" % det for det in self.detail
+            ] + [
+                SQLAlchemyError.__str__(self),
+                repr(self.statement), repr(params_repr)
+            ])
 
     def __unicode__(self):
         return self.__str__()
@@ -263,9 +276,8 @@ class DBAPIError(StatementError):
 
     @classmethod
     def instance(cls, statement, params,
-                        orig,
-                        dbapi_base_err,
-                        connection_invalidated=False):
+                 orig, dbapi_base_err,
+                 connection_invalidated=False):
         # Don't ever wrap these, just return them directly as if
         # DBAPIError didn't exist.
         if isinstance(orig, (KeyboardInterrupt, SystemExit, DontWrapMixin)):
@@ -290,21 +302,21 @@ class DBAPIError(StatementError):
 
     def __reduce__(self):
         return self.__class__, (self.statement, self.params,
-                    self.orig, self.connection_invalidated)
+                                self.orig, self.connection_invalidated)
 
     def __init__(self, statement, params, orig, connection_invalidated=False):
         try:
             text = str(orig)
         except (KeyboardInterrupt, SystemExit):
             raise
-        except Exception, e:
+        except Exception as e:
             text = 'Error in str() of DB-API-generated exception: ' + str(e)
         StatementError.__init__(
-                self,
-                '(%s) %s' % (orig.__class__.__name__, text),
-                statement,
-                params,
-                orig
+            self,
+            '(%s) %s' % (orig.__class__.__name__, text),
+            statement,
+            params,
+            orig
         )
         self.connection_invalidated = connection_invalidated
 
