@@ -1,5 +1,6 @@
 # sql/operators.py
-# Copyright (C) 2005-2013 the SQLAlchemy authors and contributors <see AUTHORS file>
+# Copyright (C) 2005-2014 the SQLAlchemy authors and contributors
+# <see AUTHORS file>
 #
 # This module is part of SQLAlchemy and is released under
 # the MIT License: http://www.opensource.org/licenses/mit-license.php
@@ -9,22 +10,25 @@
 
 """Defines operators used in SQL expressions."""
 
+from .. import util
+
+
 from operator import (
     and_, or_, inv, add, mul, sub, mod, truediv, lt, le, ne, gt, ge, eq, neg,
     getitem, lshift, rshift
-    )
+)
 
-# Py2K
-from operator import (div,)
-# end Py2K
-
-from ..util import symbol
+if util.py2k:
+    from operator import div
+else:
+    div = truediv
 
 
 class Operators(object):
     """Base of comparison and logical operators.
 
-    Implements base methods :meth:`~sqlalchemy.sql.operators.Operators.operate` and
+    Implements base methods
+    :meth:`~sqlalchemy.sql.operators.Operators.operate` and
     :meth:`~sqlalchemy.sql.operators.Operators.reverse_operate`, as well as
     :meth:`~sqlalchemy.sql.operators.Operators.__and__`,
     :meth:`~sqlalchemy.sql.operators.Operators.__or__`,
@@ -34,6 +38,7 @@ class Operators(object):
     :class:`.ColumnOperators`.
 
     """
+
     def __and__(self, other):
         """Implement the ``&`` operator.
 
@@ -99,7 +104,7 @@ class Operators(object):
         """
         return self.operate(inv)
 
-    def op(self, opstring, precedence=0):
+    def op(self, opstring, precedence=0, is_comparison=False):
         """produce a generic operator function.
 
         e.g.::
@@ -131,12 +136,23 @@ class Operators(object):
 
          .. versionadded:: 0.8 - added the 'precedence' argument.
 
+        :param is_comparison: if True, the operator will be considered as a
+         "comparison" operator, that is which evaulates to a boolean
+         true/false value, like ``==``, ``>``, etc.  This flag should be set
+         so that ORM relationships can establish that the operator is a
+         comparison operator when used in a custom join condition.
+
+         .. versionadded:: 0.9.2 - added the
+            :paramref:`.Operators.op.is_comparison` flag.
+
         .. seealso::
 
             :ref:`types_operators`
 
+            :ref:`relationship_custom_operator`
+
         """
-        operator = custom_op(opstring, precedence)
+        operator = custom_op(opstring, precedence, is_comparison)
 
         def against(other):
             return operator(self, other)
@@ -197,9 +213,10 @@ class custom_op(object):
     """
     __name__ = 'custom_op'
 
-    def __init__(self, opstring, precedence=0):
+    def __init__(self, opstring, precedence=0, is_comparison=False):
         self.opstring = opstring
         self.precedence = precedence
+        self.is_comparison = is_comparison
 
     def __eq__(self, other):
         return isinstance(other, custom_op) and \
@@ -406,8 +423,8 @@ class ColumnOperators(Operators):
     def notin_(self, other):
         """implement the ``NOT IN`` operator.
 
-        This is equivalent to using negation with :meth:`.ColumnOperators.in_`,
-        i.e. ``~x.in_(y)``.
+        This is equivalent to using negation with
+        :meth:`.ColumnOperators.in_`, i.e. ``~x.in_(y)``.
 
         .. versionadded:: 0.8
 
@@ -503,11 +520,17 @@ class ColumnOperators(Operators):
         return self.operate(contains_op, other, **kwargs)
 
     def match(self, other, **kwargs):
-        """Implements the 'match' operator.
+        """Implements a database-specific 'match' operator.
 
-        In a column context, this produces a MATCH clause, i.e.
-        ``MATCH '<other>'``.  The allowed contents of ``other``
-        are database backend specific.
+        :meth:`~.ColumnOperators.match` attempts to resolve to
+        a MATCH-like function or operator provided by the backend.
+        Examples include:
+
+        * Postgresql - renders ``x @@ to_tsquery(y)``
+        * MySQL - renders ``MATCH (x) AGAINST (y IN BOOLEAN MODE)``
+        * Oracle - renders ``CONTAINS(x, y)``
+        * other backends may provide special implementations;
+          some backends such as SQLite have no support.
 
         """
         return self.operate(match_op, other, **kwargs)
@@ -569,10 +592,12 @@ class ColumnOperators(Operators):
         """
         return self.reverse_operate(div, other)
 
-    def between(self, cleft, cright):
+    def between(self, cleft, cright, symmetric=False):
         """Produce a :func:`~.expression.between` clause against
-        the parent object, given the lower and upper range."""
-        return self.operate(between_op, cleft, cright)
+        the parent object, given the lower and upper range.
+
+        """
+        return self.operate(between_op, cleft, cright, symmetric=symmetric)
 
     def distinct(self):
         """Produce a :func:`~.expression.distinct` clause against the
@@ -654,6 +679,14 @@ def exists():
     raise NotImplementedError()
 
 
+def istrue(a):
+    raise NotImplementedError()
+
+
+def isfalse(a):
+    raise NotImplementedError()
+
+
 def is_(a, b):
     return a.is_(b)
 
@@ -686,8 +719,12 @@ def notilike_op(a, b, escape=None):
     return a.notilike(b, escape=escape)
 
 
-def between_op(a, b, c):
-    return a.between(b, c)
+def between_op(a, b, c, symmetric=False):
+    return a.between(b, c, symmetric=symmetric)
+
+
+def notbetween_op(a, b, c, symmetric=False):
+    return a.notbetween(b, c, symmetric=symmetric)
 
 
 def in_op(a, b):
@@ -726,8 +763,8 @@ def notcontains_op(a, b, escape=None):
     return ~a.contains(b, escape=escape)
 
 
-def match_op(a, b):
-    return a.match(b)
+def match_op(a, b, **kw):
+    return a.match(b, **kw)
 
 
 def comma_op(a, b):
@@ -756,11 +793,12 @@ def nullslast_op(a):
 
 _commutative = set([eq, ne, add, mul])
 
-_comparison = set([eq, ne, lt, gt, ge, le, between_op])
+_comparison = set([eq, ne, lt, gt, ge, le, between_op, like_op])
 
 
 def is_comparison(op):
-    return op in _comparison
+    return op in _comparison or \
+        isinstance(op, custom_op) and op.is_comparison
 
 
 def is_commutative(op):
@@ -769,7 +807,7 @@ def is_commutative(op):
 
 def is_ordering_modifier(op):
     return op in (asc_op, desc_op,
-                    nullsfirst_op, nullslast_op)
+                  nullsfirst_op, nullslast_op)
 
 _associative = _commutative.union([concat_op, and_, or_])
 
@@ -779,17 +817,16 @@ parenthesize (a op b).
 
 """
 
-_smallest = symbol('_smallest', canonical=-100)
-_largest = symbol('_largest', canonical=100)
+_asbool = util.symbol('_asbool', canonical=-10)
+_smallest = util.symbol('_smallest', canonical=-100)
+_largest = util.symbol('_largest', canonical=100)
 
 _PRECEDENCE = {
     from_: 15,
     getitem: 15,
     mul: 8,
     truediv: 8,
-    # Py2K
     div: 8,
-    # end Py2K
     mod: 8,
     neg: 8,
     add: 7,
@@ -816,14 +853,22 @@ _PRECEDENCE = {
     le: 5,
 
     between_op: 5,
+    notbetween_op: 5,
     distinct_op: 5,
     inv: 5,
+    istrue: 5,
+    isfalse: 5,
     and_: 3,
     or_: 2,
     comma_op: -1,
-    collate: 7,
+
+    desc_op: 3,
+    asc_op: 3,
+    collate: 4,
+
     as_: -1,
     exists: 0,
+    _asbool: -10,
     _smallest: _smallest,
     _largest: _largest
 }
@@ -834,6 +879,6 @@ def is_precedent(operator, against):
         return False
     else:
         return (_PRECEDENCE.get(operator,
-                getattr(operator, 'precedence', _smallest)) <=
-            _PRECEDENCE.get(against,
-                getattr(against, 'precedence', _largest)))
+                                getattr(operator, 'precedence', _smallest)) <=
+                _PRECEDENCE.get(against,
+                                getattr(against, 'precedence', _largest)))
