@@ -1,8 +1,16 @@
+# testing/warnings.py
+# Copyright (C) 2005-2014 the SQLAlchemy authors and contributors
+# <see AUTHORS file>
+#
+# This module is part of SQLAlchemy and is released under
+# the MIT License: http://www.opensource.org/licenses/mit-license.php
+
 from __future__ import absolute_import
 
 import warnings
 from .. import exc as sa_exc
 from .. import util
+import re
 
 
 def testing_warn(msg, stacklevel=3):
@@ -10,7 +18,7 @@ def testing_warn(msg, stacklevel=3):
 
     filename = "sqlalchemy.testing.warnings"
     lineno = 1
-    if isinstance(msg, basestring):
+    if isinstance(msg, util.string_types):
         warnings.warn_explicit(msg, sa_exc.SAWarning, filename, lineno)
     else:
         warnings.warn_explicit(msg, filename, lineno)
@@ -27,7 +35,7 @@ def resetwarnings():
     warnings.filterwarnings('error', category=sa_exc.SAWarning)
 
 
-def assert_warnings(fn, warnings):
+def assert_warnings(fn, warnings, regex=False):
     """Assert that each of the given warnings are emitted by fn."""
 
     from .assertions import eq_, emits_warning
@@ -39,7 +47,10 @@ def assert_warnings(fn, warnings):
         orig_warn(*args, **kw)
         popwarn = warnings.pop(0)
         canary.append(popwarn)
-        eq_(args[0], popwarn)
+        if regex:
+            assert re.match(popwarn, args[0])
+        else:
+            eq_(args[0], popwarn)
     util.warn = util.langhelpers.warn = capture_warnings
 
     result = emits_warning()(fn)()
