@@ -6,6 +6,7 @@ from sqlalchemy import sql, exc, schema, types as sqltypes
 from sqlalchemy.dialects.mysql import base as mysql
 from sqlalchemy.testing import fixtures, AssertsCompiledSQL
 from sqlalchemy import testing
+from sqlalchemy.sql import table, column
 
 class CompileTest(fixtures.TestBase, AssertsCompiledSQL):
 
@@ -40,6 +41,39 @@ class CompileTest(fixtures.TestBase, AssertsCompiledSQL):
         self.assert_compile(schema.CreateIndex(idx2),
             'CREATE INDEX test_idx2 ON testtbl (data(5))')
 
+    def test_create_index_with_length_quoted(self):
+        m = MetaData()
+        tbl = Table('testtbl', m, Column('some quoted data',
+                                String(255), key='s'))
+        idx1 = Index('test_idx1', tbl.c.s, mysql_length=10)
+
+        self.assert_compile(schema.CreateIndex(idx1),
+            'CREATE INDEX test_idx1 ON testtbl (`some quoted data`(10))')
+
+    def test_create_composite_index_with_length_quoted(self):
+        m = MetaData()
+        tbl = Table('testtbl', m,
+                    Column('some Quoted a', String(255), key='a'),
+                    Column('some Quoted b', String(255), key='b'))
+        idx1 = Index('test_idx1', tbl.c.a, tbl.c.b,
+                mysql_length={'some Quoted a': 10, 'some Quoted b': 20})
+
+        self.assert_compile(schema.CreateIndex(idx1),
+            'CREATE INDEX test_idx1 ON testtbl '
+            '(`some Quoted a`(10), `some Quoted b`(20))')
+
+    def test_create_composite_index_with_length_quoted_3085_workaround(self):
+        m = MetaData()
+        tbl = Table('testtbl', m,
+                    Column('some quoted a', String(255), key='a'),
+                    Column('some quoted b', String(255), key='b'))
+        idx1 = Index('test_idx1', tbl.c.a, tbl.c.b,
+                mysql_length={'`some quoted a`': 10, '`some quoted b`': 20})
+
+        self.assert_compile(schema.CreateIndex(idx1),
+            'CREATE INDEX test_idx1 ON testtbl '
+            '(`some quoted a`(10), `some quoted b`(20))')
+
     def test_create_composite_index_with_length(self):
         m = MetaData()
         tbl = Table('testtbl', m,
@@ -83,7 +117,7 @@ class CompileTest(fixtures.TestBase, AssertsCompiledSQL):
             PrimaryKeyConstraint('data'))
 
         self.assert_compile(schema.CreateTable(tbl),
-            "CREATE TABLE testtbl (data VARCHAR(255), PRIMARY KEY (data))")
+            "CREATE TABLE testtbl (data VARCHAR(255) NOT NULL, PRIMARY KEY (data))")
 
     def test_create_pk_with_using(self):
         m = MetaData()
@@ -91,7 +125,7 @@ class CompileTest(fixtures.TestBase, AssertsCompiledSQL):
             PrimaryKeyConstraint('data', mysql_using='btree'))
 
         self.assert_compile(schema.CreateTable(tbl),
-            "CREATE TABLE testtbl (data VARCHAR(255), "
+            "CREATE TABLE testtbl (data VARCHAR(255) NOT NULL, "
             "PRIMARY KEY (data) USING btree)")
 
     def test_create_index_expr(self):
@@ -104,53 +138,19 @@ class CompileTest(fixtures.TestBase, AssertsCompiledSQL):
             "CREATE INDEX bar ON foo (x > 5)"
         )
 
-    def test_deferrable_emits_warning(self):
+    def test_deferrable_initially_kw_not_ignored(self):
         m = MetaData()
         t1 = Table('t1', m, Column('id', Integer, primary_key=True))
         t2 = Table('t2', m, Column('id', Integer,
-                        ForeignKey('t1.id', deferrable=True),
+                        ForeignKey('t1.id', deferrable=True, initially="XYZ"),
                             primary_key=True))
 
-        assert_raises_message(
-            exc.SAWarning,
-            "The 'deferrable' keyword will no longer be ignored by the MySQL "
-            "backend in 0.9 - please adjust so that this keyword is not used in "
-            "conjunction with MySQL.",
-            schema.CreateTable(t2).compile, dialect=mysql.dialect()
-        )
-
-        @testing.emits_warning("The 'deferrable' keyword")
-        def go():
-            self.assert_compile(
+        self.assert_compile(
             schema.CreateTable(t2),
             "CREATE TABLE t2 (id INTEGER NOT NULL, "
-            "PRIMARY KEY (id), FOREIGN KEY(id) REFERENCES t1 (id))")
-        go()
-
-    def test_initially_emits_warning(self):
-        m = MetaData()
-        t1 = Table('t1', m, Column('id', Integer, primary_key=True))
-        t2 = Table('t2', m, Column('id', Integer,
-                        ForeignKey('t1.id', initially="XYZ"),
-                            primary_key=True))
-
-        assert_raises_message(
-            exc.SAWarning,
-            "The 'initially' keyword will no longer be ignored by the MySQL "
-            "backend in 0.9 - please adjust so that this keyword is not used "
-            "in conjunction with MySQL.",
-            schema.CreateTable(t2).compile, dialect=mysql.dialect()
+            "PRIMARY KEY (id), FOREIGN KEY(id) REFERENCES t1 (id) DEFERRABLE INITIALLY XYZ)"
         )
 
-        @testing.emits_warning("The 'initially' keyword ")
-        def go():
-            self.assert_compile(
-            schema.CreateTable(t2),
-            "CREATE TABLE t2 (id INTEGER NOT NULL, "
-            "PRIMARY KEY (id), FOREIGN KEY(id) REFERENCES t1 (id))")
-        go()
-
-
     def test_match_kw_raises(self):
         m = MetaData()
         t1 = Table('t1', m, Column('id', Integer, primary_key=True))
@@ -159,12 +159,26 @@ class CompileTest(fixtures.TestBase, AssertsCompiledSQL):
                             primary_key=True))
 
         assert_raises_message(
-            exc.SAWarning,
+            exc.CompileError,
             "MySQL ignores the 'MATCH' keyword while at the same time causes "
             "ON UPDATE/ON DELETE clauses to be ignored.",
             schema.CreateTable(t2).compile, dialect=mysql.dialect()
         )
 
+    def test_for_update(self):
+        table1 = table('mytable',
+                    column('myid'), column('name'), column('description'))
+
+        self.assert_compile(
+            table1.select(table1.c.myid == 7).with_for_update(),
+            "SELECT mytable.myid, mytable.name, mytable.description "
+            "FROM mytable WHERE mytable.myid = %s FOR UPDATE")
+
+        self.assert_compile(
+            table1.select(table1.c.myid == 7).with_for_update(read=True),
+            "SELECT mytable.myid, mytable.name, mytable.description "
+            "FROM mytable WHERE mytable.myid = %s LOCK IN SHARE MODE")
+
 class SQLTest(fixtures.TestBase, AssertsCompiledSQL):
     """Tests MySQL-dialect specific compilation."""
 
@@ -199,25 +213,6 @@ class SQLTest(fixtures.TestBase, AssertsCompiledSQL):
             gen(True, ['high_priority', sql.text('sql_cache')]),
             'SELECT high_priority sql_cache DISTINCT q')
 
-    @testing.uses_deprecated
-    def test_deprecated_distinct(self):
-        dialect = self.__dialect__
-
-        self.assert_compile(
-            select(['q'], distinct='ALL'),
-            'SELECT ALL q',
-        )
-
-        self.assert_compile(
-            select(['q'], distinct='distinctROW'),
-            'SELECT DISTINCTROW q',
-        )
-
-        self.assert_compile(
-            select(['q'], distinct='ALL',
-                    prefixes=['HIGH_PRIORITY', 'SQL_SMALL_RESULT']),
-            'SELECT HIGH_PRIORITY SQL_SMALL_RESULT ALL q'
-        )
 
     def test_backslash_escaping(self):
         self.assert_compile(
@@ -360,8 +355,10 @@ class SQLTest(fixtures.TestBase, AssertsCompiledSQL):
             (VARCHAR, "CAST(t.col AS CHAR)"),
             (NCHAR, "CAST(t.col AS CHAR)"),
             (CHAR, "CAST(t.col AS CHAR)"),
+            (m.CHAR(charset='utf8'), "CAST(t.col AS CHAR CHARACTER SET utf8)"),
             (CLOB, "CAST(t.col AS CHAR)"),
             (TEXT, "CAST(t.col AS CHAR)"),
+            (m.TEXT(charset='utf8'), "CAST(t.col AS CHAR CHARACTER SET utf8)"),
             (String(32), "CAST(t.col AS CHAR(32))"),
             (Unicode(32), "CAST(t.col AS CHAR(32))"),
             (CHAR(32), "CAST(t.col AS CHAR(32))"),
@@ -493,3 +490,33 @@ class SQLTest(fixtures.TestBase, AssertsCompiledSQL):
             'KEY idx_autoinc_order (`order`)'
             ')ENGINE=InnoDB')
 
+    def test_create_table_with_partition(self):
+        t1 = Table(
+            'testtable', MetaData(),
+            Column('id', Integer(), primary_key=True, autoincrement=True),
+            Column('other_id', Integer(), primary_key=True, autoincrement=False),
+            mysql_partitions='2', mysql_partition_by='KEY(other_id)')
+        self.assert_compile(
+            schema.CreateTable(t1),
+            'CREATE TABLE testtable ('
+            'id INTEGER NOT NULL AUTO_INCREMENT, '
+            'other_id INTEGER NOT NULL, '
+            'PRIMARY KEY (id, other_id)'
+            ')PARTITION BY KEY(other_id) PARTITIONS 2'
+        )
+
+    def test_create_table_with_partition_hash(self):
+        t1 = Table(
+            'testtable', MetaData(),
+            Column('id', Integer(), primary_key=True, autoincrement=True),
+            Column('other_id', Integer(), primary_key=True, autoincrement=False),
+            mysql_partitions='2', mysql_partition_by='HASH(other_id)')
+        self.assert_compile(
+            schema.CreateTable(t1),
+            'CREATE TABLE testtable ('
+            'id INTEGER NOT NULL AUTO_INCREMENT, '
+            'other_id INTEGER NOT NULL, '
+            'PRIMARY KEY (id, other_id)'
+            ')PARTITION BY HASH(other_id) PARTITIONS 2'
+        )
+
