@@ -1,15 +1,16 @@
-from test.lib.testing import assert_raises, assert_raises_message
+
+from sqlalchemy.testing import assert_raises, assert_raises_message
 from sqlalchemy.schema import DDL, CheckConstraint, AddConstraint, \
     DropConstraint
 from sqlalchemy import create_engine
 from sqlalchemy import MetaData, Integer, String, event, exc, text
-from test.lib.schema import Table
-from test.lib.schema import Column
+from sqlalchemy.testing.schema import Table
+from sqlalchemy.testing.schema import Column
 import sqlalchemy as tsa
-from test.lib import testing, engines
-from test.lib.testing import AssertsCompiledSQL, eq_
-from nose import SkipTest
-from test.lib import fixtures
+from sqlalchemy import testing
+from sqlalchemy.testing import engines
+from sqlalchemy.testing import AssertsCompiledSQL, eq_
+from sqlalchemy.testing import fixtures
 
 
 class DDLEventTest(fixtures.TestBase):
@@ -427,11 +428,9 @@ class DDLExecutionTest(fixtures.TestBase):
         strings = ' '.join(str(x) for x in pg_mock.mock)
         assert 'my_test_constraint' in strings
 
+    @testing.requires.sqlite
     def test_ddl_execute(self):
-        try:
-            engine = create_engine('sqlite:///')
-        except ImportError:
-            raise SkipTest('Requires sqlite')
+        engine = create_engine('sqlite:///')
         cx = engine.connect()
         table = self.users
         ddl = DDL('SELECT 1')
@@ -469,19 +468,26 @@ class DDLExecutionTest(fixtures.TestBase):
         default_from = testing.db.dialect.statement_compiler(
                             testing.db.dialect, None).default_from()
 
-        eq_(
-            testing.db.execute(
-                text("select 'foo%something'" + default_from)
-            ).scalar(),
-            'foo%something'
-        )
+        # We're abusing the DDL()
+        # construct here by pushing a SELECT through it
+        # so that we can verify the round trip.
+        # the DDL() will trigger autocommit, which prohibits
+        # some DBAPIs from returning results (pyodbc), so we
+        # run in an explicit transaction.
+        with testing.db.begin() as conn:
+            eq_(
+                conn.execute(
+                    text("select 'foo%something'" + default_from)
+                ).scalar(),
+                'foo%something'
+            )
 
-        eq_(
-            testing.db.execute(
-                DDL("select 'foo%%something'" + default_from)
-            ).scalar(),
-            'foo%something'
-        )
+            eq_(
+                conn.execute(
+                    DDL("select 'foo%%something'" + default_from)
+                ).scalar(),
+                'foo%something'
+            )
 
 
 
