@@ -42,7 +42,7 @@ class InheritTest(fixtures.MappedTest):
     def testbasic(self):
         class Principal(object):
             def __init__(self, **kwargs):
-                for key, value in kwargs.iteritems():
+                for key, value in kwargs.items():
                     setattr(self, key, value)
 
         class User(Principal):
@@ -94,8 +94,8 @@ class InheritTest2(fixtures.MappedTest):
 
         mapper(Foo, foo)
         mapper(Bar, bar, inherits=Foo)
-        print foo.join(bar).primary_key
-        print class_mapper(Bar).primary_key
+        print(foo.join(bar).primary_key)
+        print(class_mapper(Bar).primary_key)
         b = Bar('somedata')
         sess = create_session()
         sess.add(b)
@@ -133,8 +133,8 @@ class InheritTest2(fixtures.MappedTest):
         sess.expunge_all()
 
         l = sess.query(Bar).all()
-        print l[0]
-        print l[0].foos
+        print(l[0])
+        print(l[0].foos)
         self.assert_unordered_result(l, Bar,
 #            {'id':1, 'data':'barfoo', 'bid':1, 'foos':(Foo, [{'id':2,'data':'subfoo1'}, {'id':3,'data':'subfoo2'}])},
             {'id':b.id, 'data':'barfoo', 'foos':(Foo, [{'id':f1.id,'data':'subfoo1'}, {'id':f2.id,'data':'subfoo2'}])},
@@ -154,11 +154,11 @@ class InheritTest3(fixtures.MappedTest):
 
         bar = Table('bar', metadata,
             Column('id', Integer, ForeignKey('foo.id'), primary_key=True),
-            Column('data', String(20)))
+            Column('bar_data', String(20)))
 
         blub = Table('blub', metadata,
             Column('id', Integer, ForeignKey('bar.id'), primary_key=True),
-            Column('data', String(20)))
+            Column('blub_data', String(20)))
 
         bar_foo = Table('bar_foo', metadata,
             Column('bar_id', Integer, ForeignKey('bar.id')),
@@ -197,11 +197,10 @@ class InheritTest3(fixtures.MappedTest):
         compare = [repr(b)] + sorted([repr(o) for o in b.foos])
         sess.expunge_all()
         l = sess.query(Bar).all()
-        print repr(l[0]) + repr(l[0].foos)
+        print(repr(l[0]) + repr(l[0].foos))
         found = [repr(l[0])] + sorted([repr(o) for o in l[0].foos])
         eq_(found, compare)
 
-    @testing.fails_on('maxdb', 'FIXME: unknown')
     def testadvanced(self):
         class Foo(object):
             def __init__(self, data=None):
@@ -217,11 +216,13 @@ class InheritTest3(fixtures.MappedTest):
 
         class Blub(Bar):
             def __repr__(self):
-                return "Blub id %d, data %s, bars %s, foos %s" % (self.id, self.data, repr([b for b in self.bars]), repr([f for f in self.foos]))
+                return "Blub id %d, data %s, bars %s, foos %s" % (
+                        self.id, self.data, repr([b for b in self.bars]),
+                        repr([f for f in self.foos]))
 
         mapper(Blub, blub, inherits=Bar, properties={
-            'bars':relationship(Bar, secondary=blub_bar, lazy='joined'),
-            'foos':relationship(Foo, secondary=blub_foo, lazy='joined'),
+            'bars': relationship(Bar, secondary=blub_bar, lazy='joined'),
+            'foos': relationship(Foo, secondary=blub_foo, lazy='joined'),
         })
 
         sess = create_session()
@@ -239,11 +240,11 @@ class InheritTest3(fixtures.MappedTest):
         sess.expunge_all()
 
         l = sess.query(Blub).all()
-        print l
+        print(l)
         self.assert_(repr(l[0]) == compare)
         sess.expunge_all()
         x = sess.query(Blub).filter_by(id=blubid).one()
-        print x
+        print(x)
         self.assert_(repr(x) == compare)
 
 
