@@ -6,7 +6,7 @@ from sqlalchemy.testing.schema import Table, Column
 from sqlalchemy.orm import mapper, relationship, \
     create_session, class_mapper, \
     Mapper, column_property, \
-    Session, sessionmaker, attributes
+    Session, sessionmaker, attributes, configure_mappers
 from sqlalchemy.orm.instrumentation import ClassManager
 from sqlalchemy.orm import instrumentation, events
 from sqlalchemy.testing import eq_
@@ -18,8 +18,6 @@ from sqlalchemy.testing.mock import Mock, call
 
 class _RemoveListeners(object):
     def teardown(self):
-        # TODO: need to get remove() functionality
-        # going
         events.MapperEvents._clear()
         events.InstanceEvents._clear()
         events.SessionEvents._clear()
@@ -43,7 +41,8 @@ class MapperEventsTest(_RemoveListeners, _fixtures.FixtureTest):
             pass
 
         mapper(A, users)
-        mapper(B, addresses, inherits=A)
+        mapper(B, addresses, inherits=A,
+            properties={'address_id': addresses.c.id})
 
         def init_a(target, args, kwargs):
             canary.append(('init_a', target))
@@ -139,6 +138,58 @@ class MapperEventsTest(_RemoveListeners, _fixtures.FixtureTest):
              'before_update', 'after_update', 'before_delete',
              'after_delete'])
 
+    def test_insert_before_configured(self):
+        users, User = self.tables.users, self.classes.User
+
+        mapper(User, users)
+
+        canary = Mock()
+
+        event.listen(mapper, "before_configured", canary.listen1)
+        event.listen(mapper, "before_configured", canary.listen2, insert=True)
+        event.listen(mapper, "before_configured", canary.listen3)
+        event.listen(mapper, "before_configured", canary.listen4, insert=True)
+
+        configure_mappers()
+
+        eq_(
+            canary.mock_calls,
+            [call.listen4(), call.listen2(), call.listen1(), call.listen3()]
+        )
+
+    def test_insert_flags(self):
+        users, User = self.tables.users, self.classes.User
+
+        m = mapper(User, users)
+
+        canary = Mock()
+
+        arg = Mock()
+
+        event.listen(m, "before_insert", canary.listen1, )
+        event.listen(m, "before_insert", canary.listen2, insert=True)
+        event.listen(m, "before_insert", canary.listen3, propagate=True, insert=True)
+        event.listen(m, "load", canary.listen4)
+        event.listen(m, "load", canary.listen5, insert=True)
+        event.listen(m, "load", canary.listen6, propagate=True, insert=True)
+
+        u1 = User()
+        state = u1._sa_instance_state
+        m.dispatch.before_insert(arg, arg, arg)
+        m.class_manager.dispatch.load(arg, arg)
+        eq_(
+            canary.mock_calls,
+            [
+                call.listen3(arg, arg, arg.obj()),
+                call.listen2(arg, arg, arg.obj()),
+                call.listen1(arg, arg, arg.obj()),
+                call.listen6(arg.obj(), arg),
+                call.listen5(arg.obj(), arg),
+                call.listen4(arg.obj(), arg)
+            ]
+        )
+
+
     def test_merge(self):
         users, User = self.tables.users, self.classes.User
 
@@ -158,7 +209,7 @@ class MapperEventsTest(_RemoveListeners, _fixtures.FixtureTest):
         s = Session()
         u2 = s.merge(User(name='u2'))
         s.commit()
-        s.query(User).first()
+        s.query(User).order_by(User.id).first()
         eq_(canary, ['load', 'load', 'load'])
 
     def test_inheritance(self):
@@ -170,7 +221,8 @@ class MapperEventsTest(_RemoveListeners, _fixtures.FixtureTest):
             pass
 
         mapper(User, users)
-        mapper(AdminUser, addresses, inherits=User)
+        mapper(AdminUser, addresses, inherits=User,
+            properties={'address_id': addresses.c.id})
 
         canary1 = self.listen_all(User, propagate=True)
         canary2 = self.listen_all(User)
@@ -214,7 +266,8 @@ class MapperEventsTest(_RemoveListeners, _fixtures.FixtureTest):
 
         class AdminUser(User):
             pass
-        mapper(AdminUser, addresses, inherits=User)
+        mapper(AdminUser, addresses, inherits=User,
+            properties={'address_id': addresses.c.id})
         canary3 = self.listen_all(AdminUser)
 
         sess = create_session()
@@ -242,6 +295,7 @@ class MapperEventsTest(_RemoveListeners, _fixtures.FixtureTest):
             'before_update', 'after_update', 'before_delete',
             'after_delete'])
 
+
     def test_before_after_only_collection(self):
         """before_update is called on parent for collection modifications,
         after_update is called even if no columns were updated.
@@ -284,6 +338,42 @@ class MapperEventsTest(_RemoveListeners, _fixtures.FixtureTest):
         eq_(canary1, ['before_update', 'after_update'])
         eq_(canary2, [])
 
+    def test_before_after_configured_warn_on_non_mapper(self):
+        User, users = self.classes.User, self.tables.users
+
+        m1 = Mock()
+
+        mapper(User, users)
+        assert_raises_message(
+            sa.exc.SAWarning,
+            "before_configured' and 'after_configured' ORM events only "
+            "invoke with the mapper\(\) function or Mapper class as the target.",
+            event.listen, User, 'before_configured', m1
+        )
+
+        assert_raises_message(
+            sa.exc.SAWarning,
+            "before_configured' and 'after_configured' ORM events only "
+            "invoke with the mapper\(\) function or Mapper class as the target.",
+            event.listen, User, 'after_configured', m1
+        )
+
+    def test_before_after_configured(self):
+        User, users = self.classes.User, self.tables.users
+
+        m1 = Mock()
+        m2 = Mock()
+
+        mapper(User, users)
+
+        event.listen(mapper, "before_configured", m1)
+        event.listen(mapper, "after_configured", m2)
+
+        s = Session()
+        s.query(User)
+
+        eq_(m1.mock_calls, [call()])
+        eq_(m2.mock_calls, [call()])
 
     def test_retval(self):
         User, users = self.classes.User, self.tables.users
@@ -321,6 +411,41 @@ class MapperEventsTest(_RemoveListeners, _fixtures.FixtureTest):
         mapper(Address, addresses)
         eq_(canary, [User, Address])
 
+class DeclarativeEventListenTest(_RemoveListeners, fixtures.DeclarativeMappedTest):
+    run_setup_classes = "each"
+    run_deletes = None
+
+    def test_inheritance_propagate_after_config(self):
+        # test [ticket:2949]
+
+        class A(self.DeclarativeBasic):
+            __tablename__ = 'a'
+            id = Column(Integer, primary_key=True)
+
+        class B(A):
+            pass
+
+        listen = Mock()
+        event.listen(self.DeclarativeBasic, "load", listen, propagate=True)
+
+        class C(B):
+            pass
+
+        m1 = A.__mapper__.class_manager
+        m2 = B.__mapper__.class_manager
+        m3 = C.__mapper__.class_manager
+        a1 = A()
+        b1 = B()
+        c1 = C()
+        m3.dispatch.load(c1._sa_instance_state, "c")
+        m2.dispatch.load(b1._sa_instance_state, "b")
+        m1.dispatch.load(a1._sa_instance_state, "a")
+        eq_(
+            listen.mock_calls,
+            [call(c1, "c"), call(b1, "b"), call(a1, "a")]
+        )
+
+
 class DeferredMapperEventsTest(_RemoveListeners, _fixtures.FixtureTest):
     """"test event listeners against unmapped classes.
 
@@ -341,12 +466,12 @@ class DeferredMapperEventsTest(_RemoveListeners, _fixtures.FixtureTest):
                                 self.classes.User)
 
         canary = []
-        def evt(x):
+        def evt(x, y, z):
             canary.append(x)
         event.listen(User, "before_insert", evt, raw=True)
 
         m = mapper(User, users)
-        m.dispatch.before_insert(5)
+        m.dispatch.before_insert(5, 6, 7)
         eq_(canary, [5])
 
     def test_deferred_map_event_subclass_propagate(self):
@@ -396,12 +521,12 @@ class DeferredMapperEventsTest(_RemoveListeners, _fixtures.FixtureTest):
             pass
 
         canary = []
-        def evt(x):
+        def evt(x, y, z):
             canary.append(x)
         event.listen(User, "before_insert", evt, propagate=False)
 
         m = mapper(SubUser, users)
-        m.dispatch.before_insert(5)
+        m.dispatch.before_insert(5, 6, 7)
         eq_(canary, [])
 
     def test_deferred_map_event_subclass_post_mapping_propagate(self):
@@ -420,11 +545,11 @@ class DeferredMapperEventsTest(_RemoveListeners, _fixtures.FixtureTest):
         m = mapper(SubUser, users)
 
         canary = []
-        def evt(x):
+        def evt(x, y, z):
             canary.append(x)
         event.listen(User, "before_insert", evt, propagate=True, raw=True)
 
-        m.dispatch.before_insert(5)
+        m.dispatch.before_insert(5, 6, 7)
         eq_(canary, [5])
 
     def test_deferred_map_event_subclass_post_mapping_propagate_two(self):
@@ -619,21 +744,17 @@ class DeferredMapperEventsTest(_RemoveListeners, _fixtures.FixtureTest):
         class Bar(object):
             pass
 
-        listeners = instrumentation._instrumentation_factory.dispatch.\
-                            attribute_instrument.listeners
-        assert not listeners
+        dispatch = instrumentation._instrumentation_factory.dispatch
+        assert not dispatch.attribute_instrument
 
-        canary = []
-        def evt(x):
-            canary.append(x)
-        event.listen(Bar, "attribute_instrument", evt)
+        event.listen(Bar, "attribute_instrument", lambda: None)
 
-        eq_(len(listeners), 1)
+        eq_(len(dispatch.attribute_instrument), 1)
 
         del Bar
         gc_collect()
 
-        assert not listeners
+        assert not dispatch.attribute_instrument
 
 
     def test_deferred_instrument_event_subclass_propagate(self):
@@ -720,6 +841,92 @@ class LoadTest(_fixtures.FixtureTest):
         eq_(canary, ['load'])
 
 
+class RemovalTest(_fixtures.FixtureTest):
+    run_inserts = None
+
+
+    def test_attr_propagated(self):
+        User = self.classes.User
+
+        users, addresses, User = (self.tables.users,
+                                self.tables.addresses,
+                                self.classes.User)
+
+        class AdminUser(User):
+            pass
+
+        mapper(User, users)
+        mapper(AdminUser, addresses, inherits=User,
+            properties={'address_id': addresses.c.id})
+
+        fn = Mock()
+        event.listen(User.name, "set", fn, propagate=True)
+
+        au = AdminUser()
+        au.name = 'ed'
+
+        eq_(fn.call_count, 1)
+
+        event.remove(User.name, "set", fn)
+
+        au.name = 'jack'
+
+        eq_(fn.call_count, 1)
+
+    def test_unmapped_listen(self):
+        users = self.tables.users
+
+        class Foo(object):
+            pass
+
+        fn = Mock()
+
+        event.listen(Foo, "before_insert", fn, propagate=True)
+
+        class User(Foo):
+            pass
+
+        m = mapper(User, users)
+
+        u1 = User()
+        m.dispatch.before_insert(m, None, attributes.instance_state(u1))
+        eq_(fn.call_count, 1)
+
+        event.remove(Foo, "before_insert", fn)
+
+        # existing event is removed
+        m.dispatch.before_insert(m, None, attributes.instance_state(u1))
+        eq_(fn.call_count, 1)
+
+        # the _HoldEvents is also cleaned out
+        class Bar(Foo):
+            pass
+        m = mapper(Bar, users)
+        b1 = Bar()
+        m.dispatch.before_insert(m, None, attributes.instance_state(b1))
+        eq_(fn.call_count, 1)
+
+    def test_instance_event_listen_on_cls_before_map(self):
+        users = self.tables.users
+
+        fn = Mock()
+
+        class User(object):
+            pass
+
+        event.listen(User, "load", fn)
+        m = mapper(User, users)
+
+        u1 = User()
+        m.class_manager.dispatch.load(u1._sa_instance_state, "u1")
+
+        event.remove(User, "load", fn)
+
+        m.class_manager.dispatch.load(u1._sa_instance_state, "u2")
+
+        eq_(fn.mock_calls, [call(u1, "u1")])
+
+
 
 class RefreshTest(_fixtures.FixtureTest):
     run_inserts = None
@@ -1110,18 +1317,75 @@ class SessionEventsTest(_RemoveListeners, _fixtures.FixtureTest):
     def test_on_bulk_update_hook(self):
         User, users = self.classes.User, self.tables.users
 
-        sess, canary = self._listener_fixture()
+        sess = Session()
+        canary = Mock()
+
+        event.listen(sess, "after_begin", canary.after_begin)
+        event.listen(sess, "after_bulk_update", canary.after_bulk_update)
+
+        def legacy(ses, qry, ctx, res):
+            canary.after_bulk_update_legacy(ses, qry, ctx, res)
+        event.listen(sess, "after_bulk_update", legacy)
+
         mapper(User, users)
+
         sess.query(User).update({'name': 'foo'})
-        eq_(canary, ['after_begin', 'after_bulk_update'])
+
+        eq_(
+            canary.after_begin.call_count,
+            1
+        )
+        eq_(
+            canary.after_bulk_update.call_count,
+            1
+        )
+
+        upd = canary.after_bulk_update.mock_calls[0][1][0]
+        eq_(
+            upd.session,
+            sess
+        )
+        eq_(
+            canary.after_bulk_update_legacy.mock_calls,
+            [call(sess, upd.query, upd.context, upd.result)]
+        )
+
 
     def test_on_bulk_delete_hook(self):
         User, users = self.classes.User, self.tables.users
 
-        sess, canary = self._listener_fixture()
+        sess = Session()
+        canary = Mock()
+
+        event.listen(sess, "after_begin", canary.after_begin)
+        event.listen(sess, "after_bulk_delete", canary.after_bulk_delete)
+
+        def legacy(ses, qry, ctx, res):
+            canary.after_bulk_delete_legacy(ses, qry, ctx, res)
+        event.listen(sess, "after_bulk_delete", legacy)
+
         mapper(User, users)
+
         sess.query(User).delete()
-        eq_(canary, ['after_begin', 'after_bulk_delete'])
+
+        eq_(
+            canary.after_begin.call_count,
+            1
+        )
+        eq_(
+            canary.after_bulk_delete.call_count,
+            1
+        )
+
+        upd = canary.after_bulk_delete.mock_calls[0][1][0]
+        eq_(
+            upd.session,
+            sess
+        )
+        eq_(
+            canary.after_bulk_delete_legacy.mock_calls,
+            [call(sess, upd.query, upd.context, upd.result)]
+        )
 
     def test_connection_emits_after_begin(self):
         sess, canary = self._listener_fixture(bind=testing.db)
@@ -1334,7 +1598,8 @@ class MapperExtensionTest(_fixtures.FixtureTest):
             pass
 
         mapper(User, users, extension=Ext())
-        mapper(AdminUser, addresses, inherits=User)
+        mapper(AdminUser, addresses, inherits=User,
+            properties={'address_id': addresses.c.id})
 
         sess = create_session()
         am = AdminUser(name='au1', email_address='au1@e1')
@@ -1410,7 +1675,8 @@ class MapperExtensionTest(_fixtures.FixtureTest):
 
         ext = Ext()
         mapper(User, users, extension=ext)
-        mapper(AdminUser, addresses, inherits=User, extension=ext)
+        mapper(AdminUser, addresses, inherits=User, extension=ext,
+            properties={'address_id': addresses.c.id})
 
         sess = create_session()
         am = AdminUser(name="au1", email_address="au1@e1")
@@ -1550,19 +1816,13 @@ class SessionExtensionTest(_fixtures.FixtureTest):
                 log.append('after_attach')
             def after_bulk_update(
                 self,
-                session,
-                query,
-                query_context,
-                result,
+                session, query, query_context, result
                 ):
                 log.append('after_bulk_update')
 
             def after_bulk_delete(
                 self,
-                session,
-                query,
-                query_context,
-                result,
+                session, query, query_context, result
                 ):
                 log.append('after_bulk_delete')
 
