@@ -2,25 +2,37 @@
 Primary key changing capabilities and passive/non-passive cascading updates.
 
 """
-from sqlalchemy.test.testing import eq_, ne_, \
+
+from sqlalchemy.testing import eq_, ne_, \
                         assert_raises, assert_raises_message
 import sqlalchemy as sa
-from sqlalchemy.test import testing
+from sqlalchemy import testing
 from sqlalchemy import Integer, String, ForeignKey, Unicode
-from sqlalchemy.test.schema import Table, Column
+from sqlalchemy.testing.schema import Table, Column
 from sqlalchemy.orm import mapper, relationship, create_session, backref, Session
 from sqlalchemy.orm.session import make_transient
-from sqlalchemy.test.testing import eq_
-from test.orm import _base, _fixtures
+from sqlalchemy.testing import eq_
+from sqlalchemy.testing import fixtures
+from test.orm import _fixtures
 
-class NaturalPKTest(_base.MappedTest):
+def _backend_specific_fk_args():
+    if testing.requires.deferrable_fks.enabled:
+        fk_args = dict(deferrable=True, initially='deferred')
+    elif not testing.requires.on_update_cascade.enabled:
+        fk_args = dict()
+    else:
+        fk_args = dict(onupdate='cascade')
+    return fk_args
+
+class NaturalPKTest(fixtures.MappedTest):
+    # MySQL 5.5 on Windows crashes (the entire server, not the client)
+    # if you screw around with ON UPDATE CASCADE type of stuff.
+    __requires__ = 'skip_mysql_on_windows', 'on_update_or_deferrable_fks'
+    __backend__ = True
 
     @classmethod
     def define_tables(cls, metadata):
-        if testing.against('oracle'):
-            fk_args = dict(deferrable=True, initially='deferred')
-        else:
-            fk_args = dict(onupdate='cascade')
+        fk_args = _backend_specific_fk_args()
 
         users = Table('users', metadata,
             Column('username', String(50), primary_key=True),
@@ -29,35 +41,36 @@ class NaturalPKTest(_base.MappedTest):
 
         addresses = Table('addresses', metadata,
             Column('email', String(50), primary_key=True),
-            Column('username', String(50), 
+            Column('username', String(50),
                             ForeignKey('users.username', **fk_args)),
             test_needs_fk=True)
 
         items = Table('items', metadata,
             Column('itemname', String(50), primary_key=True),
-            Column('description', String(100)), 
+            Column('description', String(100)),
             test_needs_fk=True)
 
         users_to_items = Table('users_to_items', metadata,
-            Column('username', String(50), 
+            Column('username', String(50),
                                 ForeignKey('users.username', **fk_args),
                                 primary_key=True),
-            Column('itemname', String(50), 
+            Column('itemname', String(50),
                                 ForeignKey('items.itemname', **fk_args),
                                 primary_key=True),
             test_needs_fk=True)
 
     @classmethod
     def setup_classes(cls):
-        class User(_base.ComparableEntity):
+        class User(cls.Comparable):
             pass
-        class Address(_base.ComparableEntity):
+        class Address(cls.Comparable):
             pass
-        class Item(_base.ComparableEntity):
+        class Item(cls.Comparable):
             pass
 
-    @testing.resolve_artifact_names
     def test_entity(self):
+        users, User = self.tables.users, self.classes.User
+
         mapper(User, users)
 
         sess = create_session()
@@ -80,8 +93,9 @@ class NaturalPKTest(_base.MappedTest):
         u1 = sess.query(User).get('ed')
         eq_(User(username='ed', fullname='jack'), u1)
 
-    @testing.resolve_artifact_names
     def test_load_after_expire(self):
+        users, User = self.tables.users, self.classes.User
+
         mapper(User, users)
 
         sess = create_session()
@@ -94,7 +108,7 @@ class NaturalPKTest(_base.MappedTest):
         users.update(values={User.username:'jack'}).execute(username='ed')
 
         # expire/refresh works off of primary key.  the PK is gone
-        # in this case so theres no way to look it up.  criterion-
+        # in this case so there's no way to look it up.  criterion-
         # based session invalidation could solve this [ticket:911]
         sess.expire(u1)
         assert_raises(sa.orm.exc.ObjectDeletedError, getattr, u1, 'username')
@@ -103,8 +117,9 @@ class NaturalPKTest(_base.MappedTest):
         assert sess.query(User).get('jack') is None
         assert sess.query(User).get('ed').fullname == 'jack'
 
-    @testing.resolve_artifact_names
     def test_flush_new_pk_after_expire(self):
+        User, users = self.classes.User, self.tables.users
+
         mapper(User, users)
         sess = create_session()
         u1 = User(username='jack', fullname='jack')
@@ -120,16 +135,19 @@ class NaturalPKTest(_base.MappedTest):
         assert sess.query(User).get('ed').fullname == 'jack'
 
 
-    @testing.fails_on('sqlite', 'sqlite doesnt support ON UPDATE CASCADE')
-    @testing.fails_on('oracle', 'oracle doesnt support ON UPDATE CASCADE')
+    @testing.requires.on_update_cascade
     def test_onetomany_passive(self):
         self._test_onetomany(True)
 
     def test_onetomany_nonpassive(self):
         self._test_onetomany(False)
 
-    @testing.resolve_artifact_names
     def _test_onetomany(self, passive_updates):
+        users, Address, addresses, User = (self.tables.users,
+                                self.classes.Address,
+                                self.tables.addresses,
+                                self.classes.User)
+
         mapper(User, users, properties={
             'addresses':relationship(Address, passive_updates=passive_updates)
         })
@@ -157,15 +175,15 @@ class NaturalPKTest(_base.MappedTest):
         def go():
             sess.flush()
         if not passive_updates:
-            # test passive_updates=False; 
+            # test passive_updates=False;
             #load addresses, update user, update 2 addresses
-            self.assert_sql_count(testing.db, go, 4) 
+            self.assert_sql_count(testing.db, go, 4)
         else:
             # test passive_updates=True; update user
-            self.assert_sql_count(testing.db, go, 1) 
+            self.assert_sql_count(testing.db, go, 1)
         sess.expunge_all()
         assert User(username='jack', addresses=[
-                                        Address(username='jack'), 
+                                        Address(username='jack'),
                                         Address(username='jack')]) == \
                             sess.query(User).get('jack')
 
@@ -178,20 +196,59 @@ class NaturalPKTest(_base.MappedTest):
         u1 = sess.query(User).get('fred')
         eq_(User(username='fred', fullname='jack'), u1)
 
-
-    @testing.fails_on('sqlite', 'sqlite doesnt support ON UPDATE CASCADE')
-    @testing.fails_on('oracle', 'oracle doesnt support ON UPDATE CASCADE')
+    @testing.requires.on_update_cascade
     def test_manytoone_passive(self):
         self._test_manytoone(True)
 
     def test_manytoone_nonpassive(self):
         self._test_manytoone(False)
 
-    @testing.resolve_artifact_names
-    def _test_manytoone(self, passive_updates):
+    def test_manytoone_nonpassive_cold_mapping(self):
+        """test that the mapper-level m2o dependency processor
+        is set up even if the opposite side relationship
+        hasn't yet been part of a flush.
+
+        """
+        users, Address, addresses, User = (self.tables.users,
+                                self.classes.Address,
+                                self.tables.addresses,
+                                self.classes.User)
+
+        with testing.db.begin() as conn:
+            conn.execute(users.insert(),
+                username='jack', fullname='jack'
+                )
+            conn.execute(addresses.insert(),
+                email='jack1', username='jack'
+            )
+            conn.execute(addresses.insert(),
+                email='jack2', username='jack'
+                )
+
         mapper(User, users)
         mapper(Address, addresses, properties={
-            'user':relationship(User, passive_updates=passive_updates)
+            'user': relationship(User,
+                    passive_updates=False)
+        })
+
+        sess = create_session()
+        u1 = sess.query(User).first()
+        a1, a2 = sess.query(Address).all()
+        u1.username = 'ed'
+
+        def go():
+            sess.flush()
+        self.assert_sql_count(testing.db, go, 3)
+
+    def _test_manytoone(self, passive_updates):
+        users, Address, addresses, User = (self.tables.users,
+                                self.classes.Address,
+                                self.tables.addresses,
+                                self.classes.User)
+
+        mapper(User, users)
+        mapper(Address, addresses, properties={
+            'user': relationship(User, passive_updates=passive_updates)
         })
 
         sess = create_session()
@@ -223,16 +280,20 @@ class NaturalPKTest(_base.MappedTest):
         eq_([Address(username='ed'), Address(username='ed')],
                 sess.query(Address).all())
 
-    @testing.fails_on('sqlite', 'sqlite doesnt support ON UPDATE CASCADE')
-    @testing.fails_on('oracle', 'oracle doesnt support ON UPDATE CASCADE')
+
+    @testing.requires.on_update_cascade
     def test_onetoone_passive(self):
         self._test_onetoone(True)
 
     def test_onetoone_nonpassive(self):
         self._test_onetoone(False)
 
-    @testing.resolve_artifact_names
     def _test_onetoone(self, passive_updates):
+        users, Address, addresses, User = (self.tables.users,
+                                self.classes.Address,
+                                self.tables.addresses,
+                                self.classes.User)
+
         mapper(User, users, properties={
             "address":relationship(Address, passive_updates=passive_updates,
                                                 uselist=False)
@@ -266,19 +327,22 @@ class NaturalPKTest(_base.MappedTest):
         sess.expunge_all()
         eq_([Address(username='ed')], sess.query(Address).all())
 
-    @testing.fails_on('sqlite', 'sqlite doesnt support ON UPDATE CASCADE')
-    @testing.fails_on('oracle', 'oracle doesnt support ON UPDATE CASCADE')
+    @testing.requires.on_update_cascade
     def test_bidirectional_passive(self):
         self._test_bidirectional(True)
 
     def test_bidirectional_nonpassive(self):
         self._test_bidirectional(False)
 
-    @testing.resolve_artifact_names
     def _test_bidirectional(self, passive_updates):
+        users, Address, addresses, User = (self.tables.users,
+                                self.classes.Address,
+                                self.tables.addresses,
+                                self.classes.User)
+
         mapper(User, users)
         mapper(Address, addresses, properties={
-            'user':relationship(User, passive_updates=passive_updates,
+            'user': relationship(User, passive_updates=passive_updates,
                             backref='addresses')})
 
         sess = create_session()
@@ -321,20 +385,21 @@ class NaturalPKTest(_base.MappedTest):
                         sess.query(Address).all())
 
 
-    @testing.fails_on('sqlite', 'sqlite doesnt support ON UPDATE CASCADE')
-    @testing.fails_on('oracle', 'oracle doesnt support ON UPDATE CASCADE')
+    @testing.requires.on_update_cascade
     def test_manytomany_passive(self):
         self._test_manytomany(True)
 
-    # mysqldb executemany() of the association table fails to 
-    # report the correct row count
-    @testing.fails_if(lambda: testing.against('mysql') 
-                            and not testing.against('+zxjdbc'))
+    @testing.requires.non_updating_cascade
     def test_manytomany_nonpassive(self):
         self._test_manytomany(False)
 
-    @testing.resolve_artifact_names
     def _test_manytomany(self, passive_updates):
+        users, items, Item, User, users_to_items = (self.tables.users,
+                                self.tables.items,
+                                self.classes.Item,
+                                self.classes.User,
+                                self.tables.users_to_items)
+
         mapper(User, users, properties={
             'items':relationship(Item, secondary=users_to_items,
                             backref='users',
@@ -386,13 +451,19 @@ class NaturalPKTest(_base.MappedTest):
 
 class TransientExceptionTesst(_fixtures.FixtureTest):
     run_inserts = None
+    __backend__ = True
 
-    @testing.resolve_artifact_names
     def test_transient_exception(self):
         """An object that goes from a pk value to transient/pending
         doesn't count as a "pk" switch.
 
         """
+
+        users, Address, addresses, User = (self.tables.users,
+                                self.classes.Address,
+                                self.tables.addresses,
+                                self.classes.User)
+
         mapper(User, users)
         mapper(Address, addresses, properties={'user':relationship(User)})
 
@@ -416,46 +487,49 @@ class TransientExceptionTesst(_fixtures.FixtureTest):
         ne_(u1.id, None)
         eq_(sess.query(User).count(), 2)
 
-class ReversePKsTest(_base.MappedTest):
+class ReversePKsTest(fixtures.MappedTest):
     """reverse the primary keys of two entities and ensure bookkeeping
     succeeds."""
 
+    __backend__ = True
 
     @classmethod
     def define_tables(cls, metadata):
         Table(
             'user', metadata,
-            Column('code', Integer, primary_key=True),
-            Column('status', Integer, primary_key=True),
-            Column('username', Unicode(50), nullable=False),
+            Column('code', Integer, autoincrement=False, primary_key=True),
+            Column('status', Integer, autoincrement=False, primary_key=True),
+            Column('username', String(50), nullable=False),
+            test_needs_acid=True
             )
 
     @classmethod
     def setup_classes(cls):
-        class User(_base.ComparableEntity):
+        class User(cls.Comparable):
             def __init__(self, code, status, username):
                 self.code = code
                 self.status = status
                 self.username = username
 
-    @testing.resolve_artifact_names
     def test_reverse(self):
+        user, User = self.tables.user, self.classes.User
+
         PUBLISHED, EDITABLE, ARCHIVED = 1, 2, 3
 
         mapper(User, user)
 
         session = sa.orm.sessionmaker()()
 
-        a_published = User(1, PUBLISHED, u'a')
+        a_published = User(1, PUBLISHED, 'a')
         session.add(a_published)
         session.commit()
 
-        a_editable = User(1, EDITABLE, u'a')
+        a_editable = User(1, EDITABLE, 'a')
 
         session.add(a_editable)
         session.commit()
 
-        # do the switch in both directions - 
+        # do the switch in both directions -
         # one or the other should raise the error
         # based on platform dictionary ordering
         a_published.status = ARCHIVED
@@ -473,18 +547,48 @@ class ReversePKsTest(_base.MappedTest):
         assert session.query(User).get([1, PUBLISHED]) is a_published
         assert session.query(User).get([1, EDITABLE]) is a_editable
 
+    @testing.requires.savepoints
+    def test_reverse_savepoint(self):
+        user, User = self.tables.user, self.classes.User
 
-class SelfReferentialTest(_base.MappedTest):
-    # mssql, mysql don't allow 
+        PUBLISHED, EDITABLE, ARCHIVED = 1, 2, 3
+
+        mapper(User, user)
+
+        session = sa.orm.sessionmaker()()
+
+        a_published = User(1, PUBLISHED, 'a')
+        session.add(a_published)
+        session.commit()
+
+        a_editable = User(1, EDITABLE, 'a')
+
+        session.add(a_editable)
+        session.commit()
+
+        # testing #3108
+        session.begin_nested()
+
+        a_published.status = ARCHIVED
+        a_editable.status = PUBLISHED
+
+        session.commit()
+
+        session.rollback()
+        eq_(a_published.status, PUBLISHED)
+        eq_(a_editable.status, EDITABLE)
+
+class SelfReferentialTest(fixtures.MappedTest):
+    # mssql, mysql don't allow
     # ON UPDATE on self-referential keys
-    __unsupported_on__ = ('mssql','mysql') 
+    __unsupported_on__ = ('mssql', 'mysql')
+
+    __requires__ = 'on_update_or_deferrable_fks',
+    __backend__ = True
 
     @classmethod
     def define_tables(cls, metadata):
-        if testing.against('oracle'):
-            fk_args = dict(deferrable=True, initially='deferred')
-        else:
-            fk_args = dict(onupdate='cascade')
+        fk_args = _backend_specific_fk_args()
 
         Table('nodes', metadata,
               Column('name', String(50), primary_key=True),
@@ -495,11 +599,12 @@ class SelfReferentialTest(_base.MappedTest):
 
     @classmethod
     def setup_classes(cls):
-        class Node(_base.ComparableEntity):
+        class Node(cls.Comparable):
             pass
 
-    @testing.resolve_artifact_names
     def test_one_to_many_on_m2o(self):
+        Node, nodes = self.classes.Node, self.tables.nodes
+
         mapper(Node, nodes, properties={
             'children': relationship(Node,
                                  backref=sa.orm.backref('parentnode',
@@ -523,8 +628,9 @@ class SelfReferentialTest(_base.MappedTest):
              for n in sess.query(Node).filter(
                  Node.name.in_(['n11', 'n12', 'n13']))])
 
-    @testing.resolve_artifact_names
     def test_one_to_many_on_o2m(self):
+        Node, nodes = self.classes.Node, self.tables.nodes
+
         mapper(Node, nodes, properties={
             'children': relationship(Node,
                                  backref=sa.orm.backref('parentnode',
@@ -548,19 +654,19 @@ class SelfReferentialTest(_base.MappedTest):
              for n in sess.query(Node).filter(
                  Node.name.in_(['n11', 'n12', 'n13']))])
 
-    @testing.fails_on('sqlite', 'sqlite doesnt support ON UPDATE CASCADE')
-    @testing.fails_on('oracle', 'oracle doesnt support ON UPDATE CASCADE')
+    @testing.requires.on_update_cascade
     def test_many_to_one_passive(self):
         self._test_many_to_one(True)
 
     def test_many_to_one_nonpassive(self):
         self._test_many_to_one(False)
 
-    @testing.resolve_artifact_names
     def _test_many_to_one(self, passive):
+        Node, nodes = self.classes.Node, self.tables.nodes
+
         mapper(Node, nodes, properties={
-            'parentnode':relationship(Node, 
-                            remote_side=nodes.c.name, 
+            'parentnode':relationship(Node,
+                            remote_side=nodes.c.name,
                             passive_updates=passive)
             }
         )
@@ -581,13 +687,13 @@ class SelfReferentialTest(_base.MappedTest):
                   Node.name.in_(['n11', 'n12', 'n13']))])
 
 
-class NonPKCascadeTest(_base.MappedTest):
+class NonPKCascadeTest(fixtures.MappedTest):
+    __requires__ = 'skip_mysql_on_windows', 'on_update_or_deferrable_fks'
+    __backend__ = True
+
     @classmethod
     def define_tables(cls, metadata):
-        if testing.against('oracle'):
-            fk_args = dict(deferrable=True, initially='deferred')
-        else:
-            fk_args = dict(onupdate='cascade')
+        fk_args = _backend_specific_fk_args()
 
         Table('users', metadata,
             Column('id', Integer, primary_key=True,
@@ -607,21 +713,24 @@ class NonPKCascadeTest(_base.MappedTest):
 
     @classmethod
     def setup_classes(cls):
-        class User(_base.ComparableEntity):
+        class User(cls.Comparable):
             pass
-        class Address(_base.ComparableEntity):
+        class Address(cls.Comparable):
             pass
 
-    @testing.fails_on('sqlite', 'sqlite doesnt support ON UPDATE CASCADE')
-    @testing.fails_on('oracle', 'oracle doesnt support ON UPDATE CASCADE')
+    @testing.requires.on_update_cascade
     def test_onetomany_passive(self):
         self._test_onetomany(True)
 
     def test_onetomany_nonpassive(self):
         self._test_onetomany(False)
 
-    @testing.resolve_artifact_names
     def _test_onetomany(self, passive_updates):
+        User, Address, users, addresses = (self.classes.User,
+                                self.classes.Address,
+                                self.tables.users,
+                                self.tables.addresses)
+
         mapper(User, users, properties={
             'addresses':relationship(Address,
                                 passive_updates=passive_updates)})
@@ -643,7 +752,7 @@ class NonPKCascadeTest(_base.MappedTest):
         u1.username = 'ed'
         sess.flush()
         assert u1.addresses[0].username == 'ed'
-        eq_(sa.select([addresses.c.username]).execute().fetchall(), 
+        eq_(sa.select([addresses.c.username]).execute().fetchall(),
                     [('ed',), ('ed',)])
 
         sess.expunge_all()
@@ -655,14 +764,14 @@ class NonPKCascadeTest(_base.MappedTest):
         def go():
             sess.flush()
         if not passive_updates:
-            # test passive_updates=False; load addresses, 
+            # test passive_updates=False; load addresses,
             # update user, update 2 addresses
-            self.assert_sql_count(testing.db, go, 4) 
+            self.assert_sql_count(testing.db, go, 4)
         else:
              # test passive_updates=True; update user
             self.assert_sql_count(testing.db, go, 1)
         sess.expunge_all()
-        assert User(username='jack', 
+        assert User(username='jack',
                         addresses=[Address(username='jack'),
                                     Address(username='jack')]) == \
                     sess.query(User).get(u1.id)
@@ -676,30 +785,28 @@ class NonPKCascadeTest(_base.MappedTest):
         a1 = sess.query(Address).get(a1.id)
         eq_(a1.username, None)
 
-        eq_(sa.select([addresses.c.username]).execute().fetchall(), 
+        eq_(sa.select([addresses.c.username]).execute().fetchall(),
                         [(None,), (None,)])
 
         u1 = sess.query(User).get(u1.id)
         eq_(User(username='fred', fullname='jack'), u1)
 
 
-class CascadeToFKPKTest(_base.MappedTest, testing.AssertsCompiledSQL):
+class CascadeToFKPKTest(fixtures.MappedTest, testing.AssertsCompiledSQL):
     """A primary key mutation cascades onto a foreign key that is itself a
     primary key."""
+    __backend__ = True
 
     @classmethod
     def define_tables(cls, metadata):
-        if testing.against('oracle'):
-            fk_args = dict(deferrable=True, initially='deferred')
-        else:
-            fk_args = dict(onupdate='cascade')
+        fk_args = _backend_specific_fk_args()
 
         Table('users', metadata,
             Column('username', String(50), primary_key=True),
             test_needs_fk=True)
 
         Table('addresses', metadata,
-                Column('username', String(50), 
+                Column('username', String(50),
                        ForeignKey('users.username', **fk_args),
                        primary_key=True
                        ),
@@ -710,18 +817,16 @@ class CascadeToFKPKTest(_base.MappedTest, testing.AssertsCompiledSQL):
 
     @classmethod
     def setup_classes(cls):
-        class User(_base.ComparableEntity):
+        class User(cls.Comparable):
             pass
-        class Address(_base.ComparableEntity):
+        class Address(cls.Comparable):
             pass
 
-    @testing.fails_on('sqlite', 'sqlite doesnt support ON UPDATE CASCADE')
-    @testing.fails_on('oracle', 'oracle doesnt support ON UPDATE CASCADE')
+    @testing.requires.on_update_cascade
     def test_onetomany_passive(self):
         self._test_onetomany(True)
 
-    # PG etc. need passive=True to allow PK->PK cascade
-    @testing.fails_on_everything_except('sqlite', 'oracle', '+zxjdbc')
+    @testing.requires.non_updating_cascade
     def test_onetomany_nonpassive(self):
         self._test_onetomany(False)
 
@@ -731,14 +836,19 @@ class CascadeToFKPKTest(_base.MappedTest, testing.AssertsCompiledSQL):
     def test_o2m_change_nonpassive(self):
         self._test_o2m_change(False)
 
-    @testing.resolve_artifact_names
     def _test_o2m_change(self, passive_updates):
         """Change the PK of a related entity to another.
 
-        "on update cascade" is not involved here, so the mapper has 
+        "on update cascade" is not involved here, so the mapper has
         to do the UPDATE itself.
 
         """
+
+        User, Address, users, addresses = (self.classes.User,
+                                self.classes.Address,
+                                self.tables.users,
+                                self.tables.addresses)
+
         mapper(User, users, properties={
             'addresses':relationship(Address,
                             passive_updates=passive_updates)})
@@ -761,12 +871,17 @@ class CascadeToFKPKTest(_base.MappedTest, testing.AssertsCompiledSQL):
     def test_o2m_move_nonpassive(self):
         self._test_o2m_move(False)
 
-    @testing.resolve_artifact_names
     def _test_o2m_move(self, passive_updates):
         """Move the related entity to a different collection,
         changing its PK.
 
         """
+
+        User, Address, users, addresses = (self.classes.User,
+                                self.classes.Address,
+                                self.tables.users,
+                                self.tables.addresses)
+
         mapper(User, users, properties={
             'addresses':relationship(Address,
                             passive_updates=passive_updates)})
@@ -784,18 +899,20 @@ class CascadeToFKPKTest(_base.MappedTest, testing.AssertsCompiledSQL):
         u2.addresses.append(a1)
         sess.flush()
 
-    @testing.fails_on('oracle', 'oracle doesnt support ON UPDATE CASCADE '
-                                'but requires referential integrity')
-    @testing.fails_on('sqlite', 'sqlite doesnt support ON UPDATE CASCADE')
+    @testing.requires.on_update_cascade
     def test_change_m2o_passive(self):
         self._test_change_m2o(True)
 
-    @testing.fails_on_everything_except('sqlite', 'oracle', '+zxjdbc')
+    @testing.requires.non_updating_cascade
     def test_change_m2o_nonpassive(self):
         self._test_change_m2o(False)
 
-    @testing.resolve_artifact_names
     def _test_change_m2o(self, passive_updates):
+        User, Address, users, addresses = (self.classes.User,
+                                self.classes.Address,
+                                self.tables.users,
+                                self.tables.addresses)
+
         mapper(User, users)
         mapper(Address, addresses, properties={
             'user':relationship(User, passive_updates=passive_updates)
@@ -820,8 +937,12 @@ class CascadeToFKPKTest(_base.MappedTest, testing.AssertsCompiledSQL):
     def test_move_m2o_nonpassive(self):
         self._test_move_m2o(False)
 
-    @testing.resolve_artifact_names
     def _test_move_m2o(self, passive_updates):
+        User, Address, users, addresses = (self.classes.User,
+                                self.classes.Address,
+                                self.tables.users,
+                                self.tables.addresses)
+
         # tests [ticket:1856]
         mapper(User, users)
         mapper(Address, addresses, properties={
@@ -839,8 +960,12 @@ class CascadeToFKPKTest(_base.MappedTest, testing.AssertsCompiledSQL):
         sess.flush()
 
 
-    @testing.resolve_artifact_names
     def test_rowswitch_doesntfire(self):
+        User, Address, users, addresses = (self.classes.User,
+                                self.classes.Address,
+                                self.tables.users,
+                                self.tables.addresses)
+
         mapper(User, users)
         mapper(Address, addresses, properties={
             'user':relationship(User, passive_updates=True)
@@ -862,7 +987,7 @@ class CascadeToFKPKTest(_base.MappedTest, testing.AssertsCompiledSQL):
         sess.add(u2)
         sess.add(a2)
 
-        from sqlalchemy.test.assertsql import CompiledSQL
+        from sqlalchemy.testing.assertsql import CompiledSQL
 
         # test that the primary key columns of addresses are not
         # being updated as well, since this is a row switch.
@@ -877,15 +1002,20 @@ class CascadeToFKPKTest(_base.MappedTest, testing.AssertsCompiledSQL):
                     )
 
 
-    @testing.resolve_artifact_names
     def _test_onetomany(self, passive_updates):
         """Change the PK of a related entity via foreign key cascade.
 
-        For databases that require "on update cascade", the mapper 
+        For databases that require "on update cascade", the mapper
         has to identify the row by the new value, not the old, when
         it does the update.
 
         """
+
+        User, Address, users, addresses = (self.classes.User,
+                                self.classes.Address,
+                                self.tables.users,
+                                self.tables.addresses)
+
         mapper(User, users, properties={
             'addresses':relationship(Address,
                                 passive_updates=passive_updates)})
@@ -899,7 +1029,7 @@ class CascadeToFKPKTest(_base.MappedTest, testing.AssertsCompiledSQL):
         sess.flush()
         eq_(a1.username, 'ed')
         eq_(a2.username, 'ed')
-        eq_(sa.select([addresses.c.username]).execute().fetchall(), 
+        eq_(sa.select([addresses.c.username]).execute().fetchall(),
                 [('ed',), ('ed',)])
 
         u1.username = 'jack'
@@ -912,18 +1042,18 @@ class CascadeToFKPKTest(_base.MappedTest, testing.AssertsCompiledSQL):
                 [('jack',), ('jack', )])
 
 
-class JoinedInheritanceTest(_base.MappedTest):
+class JoinedInheritanceTest(fixtures.MappedTest):
     """Test cascades of pk->pk/fk on joined table inh."""
 
     # mssql doesn't allow ON UPDATE on self-referential keys
-    __unsupported_on__ = ('mssql',) 
+    __unsupported_on__ = ('mssql',)
+
+    __requires__ = 'skip_mysql_on_windows',
+    __backend__ = True
 
     @classmethod
     def define_tables(cls, metadata):
-        if testing.against('oracle'):
-            fk_args = dict(deferrable=True, initially='deferred')
-        else:
-            fk_args = dict(onupdate='cascade')
+        fk_args = _backend_specific_fk_args()
 
         Table('person', metadata,
             Column('name', String(50), primary_key=True),
@@ -934,13 +1064,13 @@ class JoinedInheritanceTest(_base.MappedTest):
             Column('name', String(50), ForeignKey('person.name', **fk_args),
                                         primary_key=True),
             Column('primary_language', String(50)),
-            Column('boss_name', String(50), 
+            Column('boss_name', String(50),
                                     ForeignKey('manager.name', **fk_args)),
                                     test_needs_fk=True
         )
 
         Table('manager', metadata,
-            Column('name', String(50), 
+            Column('name', String(50),
                                     ForeignKey('person.name', **fk_args),
                                     primary_key=True),
             Column('paperwork', String(50)),
@@ -949,42 +1079,44 @@ class JoinedInheritanceTest(_base.MappedTest):
 
     @classmethod
     def setup_classes(cls):
-        class Person(_base.ComparableEntity):
+        class Person(cls.Comparable):
             pass
         class Engineer(Person):
             pass
         class Manager(Person):
             pass
 
-    @testing.fails_on('sqlite', 'sqlite doesnt support ON UPDATE CASCADE')
-    @testing.fails_on('oracle', 'oracle doesnt support ON UPDATE CASCADE')
+    @testing.requires.on_update_cascade
     def test_pk_passive(self):
         self._test_pk(True)
 
-    # PG etc. need passive=True to allow PK->PK cascade
-    @testing.fails_on_everything_except('sqlite', 'oracle', '+zxjdbc')
+    @testing.requires.non_updating_cascade
     def test_pk_nonpassive(self):
         self._test_pk(False)
 
-    @testing.fails_on('sqlite', 'sqlite doesnt support ON UPDATE CASCADE')
-    @testing.fails_on('oracle', 'oracle doesnt support ON UPDATE CASCADE')
+    @testing.requires.on_update_cascade
     def test_fk_passive(self):
         self._test_fk(True)
 
     # PG etc. need passive=True to allow PK->PK cascade
-    @testing.fails_on_everything_except('sqlite', 'mysql+zxjdbc', 'oracle',
-                                                'postgresql+zxjdbc')
+    @testing.requires.non_updating_cascade
     def test_fk_nonpassive(self):
         self._test_fk(False)
 
-    @testing.resolve_artifact_names
     def _test_pk(self, passive_updates):
-        mapper(Person, person, polymorphic_on=person.c.type, 
+        Person, Manager, person, manager, Engineer, engineer = (self.classes.Person,
+                                self.classes.Manager,
+                                self.tables.person,
+                                self.tables.manager,
+                                self.classes.Engineer,
+                                self.tables.engineer)
+
+        mapper(Person, person, polymorphic_on=person.c.type,
                 polymorphic_identity='person',
                 passive_updates=passive_updates)
         mapper(Engineer, engineer, inherits=Person,
             polymorphic_identity='engineer', properties={
-            'boss':relationship(Manager, 
+            'boss':relationship(Manager,
                         primaryjoin=manager.c.name==engineer.c.boss_name,
                         passive_updates=passive_updates
                         )
@@ -1001,14 +1133,20 @@ class JoinedInheritanceTest(_base.MappedTest):
         e1.primary_language = 'c++'
         sess.commit()
 
-    @testing.resolve_artifact_names
     def _test_fk(self, passive_updates):
-        mapper(Person, person, polymorphic_on=person.c.type, 
+        Person, Manager, person, manager, Engineer, engineer = (self.classes.Person,
+                                self.classes.Manager,
+                                self.tables.person,
+                                self.tables.manager,
+                                self.classes.Engineer,
+                                self.tables.engineer)
+
+        mapper(Person, person, polymorphic_on=person.c.type,
                 polymorphic_identity='person',
                         passive_updates=passive_updates)
         mapper(Engineer, engineer, inherits=Person,
                         polymorphic_identity='engineer', properties={
-            'boss':relationship(Manager, 
+            'boss':relationship(Manager,
                         primaryjoin=manager.c.name==engineer.c.boss_name,
                         passive_updates=passive_updates
                         )
