@@ -1,20 +1,21 @@
-
-from test.lib.testing import eq_, assert_raises, \
+from __future__ import with_statement
+from sqlalchemy.testing import eq_, assert_raises, \
     assert_raises_message, assert_warnings
 from sqlalchemy import *
 from sqlalchemy.orm import attributes
 from sqlalchemy import exc as sa_exc, event
 from sqlalchemy.orm import exc as orm_exc
 from sqlalchemy.orm import *
-from test.lib.util import gc_collect
-from test.lib import testing
-from test.lib import fixtures
-from test.lib import engines
+from sqlalchemy.testing.util import gc_collect
+from sqlalchemy import testing
+from sqlalchemy.testing import fixtures
+from sqlalchemy.testing import engines
 from test.orm._fixtures import FixtureTest
 
 
 class SessionTransactionTest(FixtureTest):
     run_inserts = None
+    __backend__ = True
 
     def test_no_close_transaction_on_flush(self):
         User, users = self.classes.User, self.tables.users
@@ -65,7 +66,7 @@ class SessionTransactionTest(FixtureTest):
         try:
             conn = testing.db.connect()
             trans = conn.begin()
-            sess = create_session(bind=conn, autocommit=False, 
+            sess = create_session(bind=conn, autocommit=False,
                                 autoflush=True)
             u1 = User(name='u1')
             sess.add(u1)
@@ -83,6 +84,45 @@ class SessionTransactionTest(FixtureTest):
             conn.close()
             raise
 
+
+    @testing.requires.savepoints
+    def test_nested_accounting_new_items_removed(self):
+        User, users = self.classes.User, self.tables.users
+
+        mapper(User, users)
+
+        session = create_session(bind=testing.db)
+        session.begin()
+        session.begin_nested()
+        u1 = User(name='u1')
+        session.add(u1)
+        session.commit()
+        assert u1 in session
+        session.rollback()
+        assert u1 not in session
+
+    @testing.requires.savepoints
+    def test_nested_accounting_deleted_items_restored(self):
+        User, users = self.classes.User, self.tables.users
+
+        mapper(User, users)
+
+        session = create_session(bind=testing.db)
+        session.begin()
+        u1 = User(name='u1')
+        session.add(u1)
+        session.commit()
+
+        session.begin()
+        u1 = session.query(User).first()
+
+        session.begin_nested()
+        session.delete(u1)
+        session.commit()
+        assert u1 not in session
+        session.rollback()
+        assert u1 in session
+
     @testing.requires.savepoints
     def test_heavy_nesting(self):
         users = self.tables.users
@@ -111,7 +151,6 @@ class SessionTransactionTest(FixtureTest):
         User, users = self.classes.User, self.tables.users
 
         mapper(User, users)
-        users.delete().execute()
 
         s1 = create_session(bind=testing.db, autocommit=False)
         s2 = create_session(bind=testing.db, autocommit=False)
@@ -121,26 +160,6 @@ class SessionTransactionTest(FixtureTest):
 
         assert s2.query(User).all() == []
 
-    @testing.requires.savepoints
-    def test_rollback_ignores_clean_on_savepoint(self):
-        User, users = self.classes.User, self.tables.users
-
-        mapper(User, users)
-
-        s = Session(bind=testing.db)
-        u1 = User(name='u1')
-        u2 = User(name='u2')
-        s.add_all([u1, u2])
-        s.commit()
-        u1.name
-        u2.name
-        s.begin_nested()
-        u2.name = 'u2modified'
-        s.rollback()
-        assert 'name' not in u1.__dict__
-        assert 'name' not in u2.__dict__
-        eq_(u2.name, 'u2')
-
     @testing.requires.two_phase_transactions
     def test_twophase(self):
         users, Address, addresses, User = (self.tables.users,
@@ -154,7 +173,7 @@ class SessionTransactionTest(FixtureTest):
         mapper(Address, addresses)
 
         engine2 = engines.testing_engine()
-        sess = create_session(autocommit=True, autoflush=False, 
+        sess = create_session(autocommit=True, autoflush=False,
                             twophase=True)
         sess.bind_mapper(User, testing.db)
         sess.bind_mapper(Address, engine2)
@@ -318,6 +337,49 @@ class SessionTransactionTest(FixtureTest):
 
         eq_(len(sess.query(User).all()), 1)
 
+    def test_continue_flushing_on_commit(self):
+        """test that post-flush actions get flushed also if
+        we're in commit()"""
+        users, User = self.tables.users, self.classes.User
+
+        mapper(User, users)
+        sess = Session()
+
+        to_flush = [User(name='ed'), User(name='jack'), User(name='wendy')]
+        @event.listens_for(sess, "after_flush_postexec")
+        def add_another_user(session, ctx):
+            if to_flush:
+                session.add(to_flush.pop(0))
+
+        x = [1]
+        @event.listens_for(sess, "after_commit")
+        def add_another_user(session):
+            x[0] += 1
+
+        sess.add(to_flush.pop())
+        sess.commit()
+        eq_(x, [2])
+        eq_(
+            sess.scalar(select([func.count(users.c.id)])), 3
+        )
+
+    def test_continue_flushing_guard(self):
+        users, User = self.tables.users, self.classes.User
+
+        mapper(User, users)
+        sess = Session()
+
+        @event.listens_for(sess, "after_flush_postexec")
+        def add_another_user(session, ctx):
+            session.add(User(name='x'))
+        sess.add(User(name='x'))
+        assert_raises_message(
+            orm_exc.FlushError,
+            "Over 100 subsequent flushes have occurred",
+            sess.commit
+        )
+
+
     def test_error_on_using_inactive_session_commands(self):
         users, User = self.tables.users, self.classes.User
 
@@ -336,6 +398,80 @@ class SessionTransactionTest(FixtureTest):
                               sess.begin, subtransactions=True)
         sess.close()
 
+    def test_no_sql_during_commit(self):
+        sess = create_session(bind=testing.db, autocommit=False)
+
+        @event.listens_for(sess, "after_commit")
+        def go(session):
+            session.execute("select 1")
+        assert_raises_message(sa_exc.InvalidRequestError,
+                    "This session is in 'committed' state; no further "
+                    "SQL can be emitted within this transaction.",
+                    sess.commit)
+
+    def test_no_sql_during_prepare(self):
+        sess = create_session(bind=testing.db, autocommit=False, twophase=True)
+
+        sess.prepare()
+
+        assert_raises_message(sa_exc.InvalidRequestError,
+                    "This session is in 'prepared' state; no further "
+                    "SQL can be emitted within this transaction.",
+                    sess.execute, "select 1")
+
+    def test_no_prepare_wo_twophase(self):
+        sess = create_session(bind=testing.db, autocommit=False)
+
+        assert_raises_message(sa_exc.InvalidRequestError,
+                    "'twophase' mode not enabled, or not root "
+                    "transaction; can't prepare.",
+                    sess.prepare)
+
+    def test_closed_status_check(self):
+        sess = create_session()
+        trans = sess.begin()
+        trans.rollback()
+        assert_raises_message(
+                sa_exc.ResourceClosedError,
+                "This transaction is closed",
+                trans.rollback
+        )
+        assert_raises_message(
+                sa_exc.ResourceClosedError,
+                "This transaction is closed",
+                trans.commit
+        )
+
+    def test_deactive_status_check(self):
+        sess = create_session()
+        trans = sess.begin()
+        trans2 = sess.begin(subtransactions=True)
+        trans2.rollback()
+        assert_raises_message(
+            sa_exc.InvalidRequestError,
+            "This Session's transaction has been rolled back by a nested "
+            "rollback\(\) call.  To begin a new transaction, issue "
+            "Session.rollback\(\) first.",
+            trans.commit
+        )
+
+    def test_deactive_status_check_w_exception(self):
+        sess = create_session()
+        trans = sess.begin()
+        trans2 = sess.begin(subtransactions=True)
+        try:
+            raise Exception("test")
+        except:
+            trans2.rollback(_capture_exception=True)
+        assert_raises_message(
+            sa_exc.InvalidRequestError,
+            "This Session's transaction has been rolled back due to a "
+            "previous exception during flush. To begin a new transaction "
+            "with this Session, first issue Session.rollback\(\). "
+            "Original exception was: test",
+            trans.commit
+        )
+
     def _inactive_flushed_session_fixture(self):
         users, User = self.tables.users, self.classes.User
 
@@ -359,7 +495,7 @@ class SessionTransactionTest(FixtureTest):
         sess.add(u2)
         def go():
             sess.rollback()
-        assert_warnings(go, 
+        assert_warnings(go,
             ["Session's state has been changed on a "
             "non-active transaction - this state "
             "will be discarded."],
@@ -372,7 +508,7 @@ class SessionTransactionTest(FixtureTest):
         u1.name = 'newname'
         def go():
             sess.rollback()
-        assert_warnings(go, 
+        assert_warnings(go,
             ["Session's state has been changed on a "
             "non-active transaction - this state "
             "will be discarded."],
@@ -385,7 +521,7 @@ class SessionTransactionTest(FixtureTest):
         sess.delete(u1)
         def go():
             sess.rollback()
-        assert_warnings(go, 
+        assert_warnings(go,
             ["Session's state has been changed on a "
             "non-active transaction - this state "
             "will be discarded."],
@@ -433,7 +569,7 @@ class _LocalFixture(FixtureTest):
         users, addresses = cls.tables.users, cls.tables.addresses
         mapper(User, users, properties={
             'addresses':relationship(Address, backref='user',
-                                 cascade="all, delete-orphan", 
+                                 cascade="all, delete-orphan",
                                     order_by=addresses.c.id),
             })
         mapper(Address, addresses)
@@ -441,6 +577,7 @@ class _LocalFixture(FixtureTest):
 
 class FixtureDataTest(_LocalFixture):
     run_inserts = 'each'
+    __backend__ = True
 
     def test_attrs_on_rollback(self):
         User = self.classes.User
@@ -474,7 +611,120 @@ class FixtureDataTest(_LocalFixture):
 
         assert u1.name == 'will'
 
+class CleanSavepointTest(FixtureTest):
+    """test the behavior for [ticket:2452] - rollback on begin_nested()
+    only expires objects tracked as being modified in that transaction.
+
+    """
+    run_inserts = None
+    __backend__ = True
+
+    def _run_test(self, update_fn):
+        User, users = self.classes.User, self.tables.users
+
+        mapper(User, users)
+
+        s = Session(bind=testing.db)
+        u1 = User(name='u1')
+        u2 = User(name='u2')
+        s.add_all([u1, u2])
+        s.commit()
+        u1.name
+        u2.name
+        s.begin_nested()
+        update_fn(s, u2)
+        eq_(u2.name, 'u2modified')
+        s.rollback()
+        eq_(u1.__dict__['name'], 'u1')
+        assert 'name' not in u2.__dict__
+        eq_(u2.name, 'u2')
+
+    @testing.requires.savepoints
+    def test_rollback_ignores_clean_on_savepoint(self):
+        User, users = self.classes.User, self.tables.users
+        def update_fn(s, u2):
+            u2.name = 'u2modified'
+        self._run_test(update_fn)
+
+    @testing.requires.savepoints
+    def test_rollback_ignores_clean_on_savepoint_agg_upd_eval(self):
+        User, users = self.classes.User, self.tables.users
+        def update_fn(s, u2):
+            s.query(User).filter_by(name='u2').update(dict(name='u2modified'),
+                                    synchronize_session='evaluate')
+        self._run_test(update_fn)
+
+    @testing.requires.savepoints
+    def test_rollback_ignores_clean_on_savepoint_agg_upd_fetch(self):
+        User, users = self.classes.User, self.tables.users
+        def update_fn(s, u2):
+            s.query(User).filter_by(name='u2').update(dict(name='u2modified'),
+                                    synchronize_session='fetch')
+        self._run_test(update_fn)
+
+class ContextManagerTest(FixtureTest):
+    run_inserts = None
+    __backend__ = True
+
+    @testing.requires.savepoints
+    @engines.close_open_connections
+    def test_contextmanager_nested_rollback(self):
+        users, User = self.tables.users, self.classes.User
+
+        mapper(User, users)
+
+        sess = Session()
+        def go():
+            with sess.begin_nested():
+                sess.add(User())   # name can't be null
+                sess.flush()
+
+        # and not InvalidRequestError
+        assert_raises(
+            sa_exc.DBAPIError,
+            go
+        )
+
+        with sess.begin_nested():
+            sess.add(User(name='u1'))
+
+        eq_(sess.query(User).count(), 1)
+
+    def test_contextmanager_commit(self):
+        users, User = self.tables.users, self.classes.User
+
+        mapper(User, users)
+
+        sess = Session(autocommit=True)
+        with sess.begin():
+            sess.add(User(name='u1'))
+
+        sess.rollback()
+        eq_(sess.query(User).count(), 1)
+
+    def test_contextmanager_rollback(self):
+        users, User = self.tables.users, self.classes.User
+
+        mapper(User, users)
+
+        sess = Session(autocommit=True)
+        def go():
+            with sess.begin():
+                sess.add(User())  # name can't be null
+        assert_raises(
+            sa_exc.DBAPIError,
+            go
+        )
+
+        eq_(sess.query(User).count(), 0)
+
+        with sess.begin():
+            sess.add(User(name='u1'))
+        eq_(sess.query(User).count(), 1)
+
+
 class AutoExpireTest(_LocalFixture):
+    __backend__ = True
 
     def test_expunge_pending_on_rollback(self):
         User = self.classes.User
@@ -515,6 +765,7 @@ class AutoExpireTest(_LocalFixture):
         assert u1 in s
         assert u1 not in s.deleted
 
+    @testing.requires.predictable_gc
     def test_gced_delete_on_rollback(self):
         User, users = self.classes.User, self.tables.users
 
@@ -585,7 +836,7 @@ class AutoExpireTest(_LocalFixture):
         u1.addresses.remove(a1)
 
         s.flush()
-        eq_(s.query(Address).filter(Address.email_address=='foo').all(), 
+        eq_(s.query(Address).filter(Address.email_address=='foo').all(),
                 [])
         s.rollback()
         assert a1 not in s.deleted
@@ -616,6 +867,7 @@ class AutoExpireTest(_LocalFixture):
         assert u1.name == 'will'
 
 class TwoPhaseTest(_LocalFixture):
+    __backend__ = True
 
     @testing.requires.two_phase_transactions
     def test_rollback_on_prepare(self):
@@ -630,6 +882,7 @@ class TwoPhaseTest(_LocalFixture):
         assert u not in s
 
 class RollbackRecoverTest(_LocalFixture):
+    __backend__ = True
 
     def test_pk_violation(self):
         User, Address = self.classes.User, self.classes.Address
@@ -645,7 +898,7 @@ class RollbackRecoverTest(_LocalFixture):
         u1.name = 'edward'
         a1.email_address = 'foober'
         s.add(u2)
-        assert_raises(sa_exc.FlushError, s.commit)
+        assert_raises(orm_exc.FlushError, s.commit)
         assert_raises(sa_exc.InvalidRequestError, s.commit)
         s.rollback()
         assert u2 not in s
@@ -659,7 +912,7 @@ class RollbackRecoverTest(_LocalFixture):
         s.commit()
         eq_(
             s.query(User).all(),
-            [User(id=1, name='edward', 
+            [User(id=1, name='edward',
                 addresses=[Address(email_address='foober')])]
         )
 
@@ -679,7 +932,7 @@ class RollbackRecoverTest(_LocalFixture):
         a1.email_address = 'foober'
         s.begin_nested()
         s.add(u2)
-        assert_raises(sa_exc.FlushError, s.commit)
+        assert_raises(orm_exc.FlushError, s.commit)
         assert_raises(sa_exc.InvalidRequestError, s.commit)
         s.rollback()
         assert u2 not in s
@@ -690,12 +943,13 @@ class RollbackRecoverTest(_LocalFixture):
         s.commit()
         eq_(
             s.query(User).all(),
-            [User(id=1, name='edward', 
+            [User(id=1, name='edward',
                 addresses=[Address(email_address='foober')])]
         )
 
 
 class SavepointTest(_LocalFixture):
+    __backend__ = True
 
     @testing.requires.savepoints
     def test_savepoint_rollback(self):
@@ -711,17 +965,17 @@ class SavepointTest(_LocalFixture):
         u1.name = 'edward'
         u2.name = 'jackward'
         s.add_all([u3, u4])
-        eq_(s.query(User.name).order_by(User.id).all(), 
+        eq_(s.query(User.name).order_by(User.id).all(),
                     [('edward',), ('jackward',), ('wendy',), ('foo',)])
         s.rollback()
         assert u1.name == 'ed'
         assert u2.name == 'jack'
-        eq_(s.query(User.name).order_by(User.id).all(), 
+        eq_(s.query(User.name).order_by(User.id).all(),
                     [('ed',), ('jack',)])
         s.commit()
         assert u1.name == 'ed'
         assert u2.name == 'jack'
-        eq_(s.query(User.name).order_by(User.id).all(), 
+        eq_(s.query(User.name).order_by(User.id).all(),
                     [('ed',), ('jack',)])
 
     @testing.requires.savepoints
@@ -752,18 +1006,18 @@ class SavepointTest(_LocalFixture):
         u1.name = 'edward'
         u2.name = 'jackward'
         s.add_all([u3, u4])
-        eq_(s.query(User.name).order_by(User.id).all(), 
+        eq_(s.query(User.name).order_by(User.id).all(),
                 [('edward',), ('jackward',), ('wendy',), ('foo',)])
         s.commit()
         def go():
             assert u1.name == 'edward'
             assert u2.name == 'jackward'
-            eq_(s.query(User.name).order_by(User.id).all(), 
+            eq_(s.query(User.name).order_by(User.id).all(),
                     [('edward',), ('jackward',), ('wendy',), ('foo',)])
         self.assert_sql_count(testing.db, go, 1)
 
         s.commit()
-        eq_(s.query(User.name).order_by(User.id).all(), 
+        eq_(s.query(User.name).order_by(User.id).all(),
                 [('edward',), ('jackward',), ('wendy',), ('foo',)])
 
     @testing.requires.savepoints
@@ -781,7 +1035,7 @@ class SavepointTest(_LocalFixture):
         s.add(u2)
         eq_(s.query(User).order_by(User.id).all(),
             [
-                User(name='edward', addresses=[Address(email_address='foo'), 
+                User(name='edward', addresses=[Address(email_address='foo'),
                                         Address(email_address='bar')]),
                 User(name='jack', addresses=[Address(email_address='bat')])
             ]
@@ -789,14 +1043,14 @@ class SavepointTest(_LocalFixture):
         s.rollback()
         eq_(s.query(User).order_by(User.id).all(),
             [
-                User(name='edward', addresses=[Address(email_address='foo'), 
+                User(name='edward', addresses=[Address(email_address='foo'),
                                         Address(email_address='bar')]),
             ]
         )
         s.commit()
         eq_(s.query(User).order_by(User.id).all(),
             [
-                User(name='edward', addresses=[Address(email_address='foo'), 
+                User(name='edward', addresses=[Address(email_address='foo'),
                                         Address(email_address='bar')]),
             ]
         )
@@ -864,6 +1118,8 @@ class SavepointTest(_LocalFixture):
 
 
 class AccountingFlagsTest(_LocalFixture):
+    __backend__ = True
+
     def test_no_expire_on_commit(self):
         User, users = self.classes.User, self.tables.users
 
@@ -920,7 +1176,7 @@ class AccountingFlagsTest(_LocalFixture):
     def test_preflush_no_accounting(self):
         User, users = self.classes.User, self.tables.users
 
-        sess = Session(_enable_transaction_accounting=False, 
+        sess = Session(_enable_transaction_accounting=False,
                         autocommit=True, autoflush=False)
         u1 = User(name='ed')
         sess.add(u1)
@@ -938,6 +1194,8 @@ class AccountingFlagsTest(_LocalFixture):
 
 
 class AutoCommitTest(_LocalFixture):
+    __backend__ = True
+
     def test_begin_nested_requires_trans(self):
         sess = create_session(autocommit=True)
         assert_raises(sa_exc.InvalidRequestError, sess.begin_nested)
@@ -1039,6 +1297,8 @@ class AutoCommitTest(_LocalFixture):
         eq_(u1.id, 3)
 
 class NaturalPKRollbackTest(fixtures.MappedTest):
+    __backend__ = True
+
     @classmethod
     def define_tables(cls, metadata):
         Table('users', metadata,
@@ -1080,7 +1340,89 @@ class NaturalPKRollbackTest(fixtures.MappedTest):
 
         session.rollback()
 
+    def test_key_replaced_by_update(self):
+        users, User = self.tables.users, self.classes.User
 
+        mapper(User, users)
 
+        u1 = User(name='u1')
+        u2 = User(name='u2')
+
+        s = Session()
+        s.add_all([u1, u2])
+        s.commit()
+
+        s.delete(u1)
+        s.flush()
+
+        u2.name = 'u1'
+        s.flush()
+
+        assert u1 not in s
+        s.rollback()
+
+        assert u1 in s
+        assert u2 in s
+
+        assert s.identity_map[(User, ('u1',))] is u1
+        assert s.identity_map[(User, ('u2',))] is u2
+
+    def test_multiple_key_replaced_by_update(self):
+        users, User = self.tables.users, self.classes.User
+
+        mapper(User, users)
+
+        u1 = User(name='u1')
+        u2 = User(name='u2')
+        u3 = User(name='u3')
+
+        s = Session()
+        s.add_all([u1, u2, u3])
+        s.commit()
+
+        s.delete(u1)
+        s.delete(u2)
+        s.flush()
+
+        u3.name = 'u1'
+        s.flush()
+
+        u3.name = 'u2'
+        s.flush()
+
+        s.rollback()
+
+        assert u1 in s
+        assert u2 in s
+        assert u3 in s
+
+        assert s.identity_map[(User, ('u1',))] is u1
+        assert s.identity_map[(User, ('u2',))] is u2
+        assert s.identity_map[(User, ('u3',))] is u3
+
+    def test_key_replaced_by_oob_insert(self):
+        users, User = self.tables.users, self.classes.User
+
+        mapper(User, users)
+
+        u1 = User(name='u1')
+
+        s = Session()
+        s.add(u1)
+        s.commit()
+
+        s.delete(u1)
+        s.flush()
+
+        s.execute(users.insert().values(name='u1'))
+        u2 = s.query(User).get('u1')
+
+        assert u1 not in s
+        s.rollback()
+
+        assert u1 in s
+        assert u2 not in s
+
+        assert s.identity_map[(User, ('u1',))] is u1
 
 
