@@ -6,6 +6,7 @@ import datetime
 from sqlalchemy.orm import mapper as orm_mapper
 
 import sqlalchemy as sa
+from sqlalchemy.util import u, ue, b
 from sqlalchemy import Integer, String, ForeignKey, literal_column, event
 from sqlalchemy.testing import engines
 from sqlalchemy import testing
@@ -16,7 +17,6 @@ from sqlalchemy.orm import mapper, relationship, create_session, \
 from sqlalchemy.testing import fixtures
 from test.orm import _fixtures
 from sqlalchemy.testing.assertsql import AllOf, CompiledSQL
-from sqlalchemy import testing, util
 
 class UnitOfWorkTest(object):
     pass
@@ -88,7 +88,7 @@ class UnicodeTest(fixtures.MappedTest):
 
         mapper(Test, uni_t1)
 
-        txt = u"\u0160\u0110\u0106\u010c\u017d"
+        txt = ue("\u0160\u0110\u0106\u010c\u017d")
         t1 = Test(id=1, txt=txt)
         self.assert_(t1.txt == txt)
 
@@ -108,7 +108,7 @@ class UnicodeTest(fixtures.MappedTest):
             't2s': relationship(Test2)})
         mapper(Test2, uni_t2)
 
-        txt = u"\u0160\u0110\u0106\u010c\u017d"
+        txt = ue("\u0160\u0110\u0106\u010c\u017d")
         t1 = Test(txt=txt)
         t1.t2s.append(Test2())
         t1.t2s.append(Test2())
@@ -126,23 +126,19 @@ class UnicodeSchemaTest(fixtures.MappedTest):
 
     run_dispose_bind = 'once'
 
-    @classmethod
-    def create_engine(cls):
-        return engines.utf8_engine()
-
     @classmethod
     def define_tables(cls, metadata):
         t1 = Table('unitable1', metadata,
-              Column(u'méil', Integer, primary_key=True, key='a', test_needs_autoincrement=True),
-              Column(u'\u6e2c\u8a66', Integer, key='b'),
+              Column(u('méil'), Integer, primary_key=True, key='a', test_needs_autoincrement=True),
+              Column(ue('\u6e2c\u8a66'), Integer, key='b'),
               Column('type',  String(20)),
               test_needs_fk=True,
               test_needs_autoincrement=True)
-        t2 = Table(u'Unitéble2', metadata,
-              Column(u'méil', Integer, primary_key=True, key="cc", test_needs_autoincrement=True),
-              Column(u'\u6e2c\u8a66', Integer,
-                     ForeignKey(u'unitable1.a'), key="d"),
-              Column(u'\u6e2c\u8a66_2', Integer, key="e"),
+        t2 = Table(u('Unitéble2'), metadata,
+              Column(u('méil'), Integer, primary_key=True, key="cc", test_needs_autoincrement=True),
+              Column(ue('\u6e2c\u8a66'), Integer,
+                     ForeignKey('unitable1.a'), key="d"),
+              Column(ue('\u6e2c\u8a66_2'), Integer, key="e"),
               test_needs_fk=True,
               test_needs_autoincrement=True)
 
@@ -159,9 +155,6 @@ class UnicodeSchemaTest(fixtures.MappedTest):
 
     @testing.fails_on('mssql+pyodbc',
                       'pyodbc returns a non unicode encoding of the results description.')
-    @testing.skip_if(lambda: util.pypy,
-            "pypy/sqlite3 reports unicode cursor.description "
-            "incorrectly pre 2.2, workaround applied in 0.9")
     def test_mapping(self):
         t2, t1 = self.tables.t2, self.tables.t1
 
@@ -201,9 +194,6 @@ class UnicodeSchemaTest(fixtures.MappedTest):
 
     @testing.fails_on('mssql+pyodbc',
                       'pyodbc returns a non unicode encoding of the results description.')
-    @testing.skip_if(lambda: util.pypy,
-            "pypy/sqlite3 reports unicode cursor.description "
-            "incorrectly pre 2.2, workaround applied in 0.9")
     def test_inheritance_mapping(self):
         t2, t1 = self.tables.t2, self.tables.t1
 
@@ -244,12 +234,7 @@ class BinaryHistTest(fixtures.MappedTest, testing.AssertsExecutionResults):
     def test_binary_equality(self):
         Foo, t1 = self.classes.Foo, self.tables.t1
 
-
-        # Py3K
-        #data = b"this is some data"
-        # Py2K
-        data = "this is some data"
-        # end Py2K
+        data = b("this is some data")
 
         mapper(Foo, t1)
 
@@ -577,6 +562,7 @@ class BatchDeleteIgnoresRowcountTest(fixtures.DeclarativeMappedTest):
         class A(cls.DeclarativeBasic):
             __tablename__ = 'A'
             __table_args__ = dict(test_needs_fk=True)
+            __mapper_args__ = {"confirm_deleted_rows": False}
             id = Column(Integer, primary_key=True)
             parent_id = Column(Integer, ForeignKey('A.id', ondelete='CASCADE'))
 
@@ -584,7 +570,7 @@ class BatchDeleteIgnoresRowcountTest(fixtures.DeclarativeMappedTest):
         A = self.classes.A
         session = Session(testing.db)
 
-        a1, a2 = A(id=1),A(id=2, parent_id=1)
+        a1, a2 = A(id=1), A(id=2, parent_id=1)
 
         session.add_all([a1, a2])
         session.flush()
@@ -860,21 +846,40 @@ class DefaultTest(fixtures.MappedTest):
         eq_(h5.foober, 'im the new foober')
 
     @testing.fails_on('firebird', 'Data type unknown on the parameter')
+    @testing.fails_on("oracle+cx_oracle", "seems like a cx_oracle bug")
     def test_eager_defaults(self):
         hohoval, default_t, Hoho = (self.other.hohoval,
                                 self.tables.default_t,
                                 self.classes.Hoho)
+        Secondary = self.classes.Secondary
 
-        mapper(Hoho, default_t, eager_defaults=True)
+        mapper(Hoho, default_t, eager_defaults=True, properties={
+                "sec": relationship(Secondary),
+                "syn": sa.orm.synonym(default_t.c.counter)
+            })
 
+
+        mapper(Secondary, self.tables.secondary_table)
         h1 = Hoho()
 
         session = create_session()
         session.add(h1)
-        session.flush()
+
+        if testing.db.dialect.implicit_returning:
+            self.sql_count_(1, session.flush)
+        else:
+            self.sql_count_(2, session.flush)
 
         self.sql_count_(0, lambda: eq_(h1.hoho, hohoval))
 
+        # no actual eager defaults, make sure error isn't raised
+        h2 = Hoho(hoho=hohoval, counter=5)
+        session.add(h2)
+        session.flush()
+        eq_(h2.hoho, hohoval)
+        eq_(h2.counter, 5)
+
+
     def test_insert_nopostfetch(self):
         default_t, Hoho = self.tables.default_t, self.classes.Hoho
 
@@ -1061,13 +1066,13 @@ class OneToManyTest(_fixtures.FixtureTest):
         session.flush()
 
         user_rows = users.select(users.c.id.in_([u.id])).execute().fetchall()
-        eq_(user_rows[0].values(), [u.id, 'one2manytester'])
+        eq_(list(user_rows[0].values()), [u.id, 'one2manytester'])
 
         address_rows = addresses.select(
             addresses.c.id.in_([a.id, a2.id]),
             order_by=[addresses.c.email_address]).execute().fetchall()
-        eq_(address_rows[0].values(), [a2.id, u.id, 'lala@test.org'])
-        eq_(address_rows[1].values(), [a.id, u.id, 'one2many@test.org'])
+        eq_(list(address_rows[0].values()), [a2.id, u.id, 'lala@test.org'])
+        eq_(list(address_rows[1].values()), [a.id, u.id, 'one2many@test.org'])
 
         userid = u.id
         addressid = a2.id
@@ -1078,7 +1083,7 @@ class OneToManyTest(_fixtures.FixtureTest):
 
         address_rows = addresses.select(
             addresses.c.id == addressid).execute().fetchall()
-        eq_(address_rows[0].values(),
+        eq_(list(address_rows[0].values()),
             [addressid, userid, 'somethingnew@foo.com'])
         self.assert_(u.id == userid and a2.id == addressid)
 
@@ -1305,7 +1310,7 @@ class SaveTest(_fixtures.FixtureTest):
         session.add_all((u, u2))
         session.flush()
 
-        # assert the first one retreives the same from the identity map
+        # assert the first one retrieves the same from the identity map
         nu = session.query(m).get(u.id)
         assert u is nu
 
@@ -1403,7 +1408,9 @@ class SaveTest(_fixtures.FixtureTest):
 
         # define a mapper for AddressUser that inherits the User.mapper, and
         # joins on the id column
-        mapper(AddressUser, addresses, inherits=m1)
+        mapper(AddressUser, addresses, inherits=m1, properties={
+                'address_id': addresses.c.id
+            })
 
         au = AddressUser(name='u', email_address='u@e')
 
@@ -1425,7 +1432,7 @@ class SaveTest(_fixtures.FixtureTest):
         mapper(Order, orders, properties={
             'description': sa.orm.deferred(orders.c.description)})
 
-        # dont set deferred attribute, commit session
+        # don't set deferred attribute, commit session
         o = Order(id=42)
         session = create_session(autocommit=False)
         session.add(o)
@@ -1508,18 +1515,18 @@ class SaveTest(_fixtures.FixtureTest):
         assert u.name == 'multitester'
 
         user_rows = users.select(users.c.id.in_([u.foo_id])).execute().fetchall()
-        eq_(user_rows[0].values(), [u.foo_id, 'multitester'])
+        eq_(list(user_rows[0].values()), [u.foo_id, 'multitester'])
         address_rows = addresses.select(addresses.c.id.in_([u.id])).execute().fetchall()
-        eq_(address_rows[0].values(), [u.id, u.foo_id, 'multi@test.org'])
+        eq_(list(address_rows[0].values()), [u.id, u.foo_id, 'multi@test.org'])
 
         u.email = 'lala@hey.com'
         u.name = 'imnew'
         session.flush()
 
         user_rows = users.select(users.c.id.in_([u.foo_id])).execute().fetchall()
-        eq_(user_rows[0].values(), [u.foo_id, 'imnew'])
+        eq_(list(user_rows[0].values()), [u.foo_id, 'imnew'])
         address_rows = addresses.select(addresses.c.id.in_([u.id])).execute().fetchall()
-        eq_(address_rows[0].values(), [u.id, u.foo_id, 'lala@hey.com'])
+        eq_(list(address_rows[0].values()), [u.id, u.foo_id, 'lala@hey.com'])
 
         session.expunge_all()
         u = session.query(User).get(id)
@@ -1657,7 +1664,7 @@ class ManyToOneTest(_fixtures.FixtureTest):
         l = sa.select([users, addresses],
                       sa.and_(users.c.id==addresses.c.user_id,
                               addresses.c.id==a.id)).execute()
-        eq_(l.first().values(),
+        eq_(list(l.first().values()),
             [a.user.id, 'asdf8d', a.id, a.user_id, 'theater@foo.com'])
 
     def test_many_to_one_1(self):
@@ -2134,7 +2141,6 @@ class SaveTest3(fixtures.MappedTest):
 
         assert assoc.count().scalar() == 2
         i.keywords = []
-        print i.keywords
         session.flush()
         assert assoc.count().scalar() == 0
 
@@ -2340,12 +2346,12 @@ class InheritingRowSwitchTest(fixtures.MappedTest):
     @classmethod
     def define_tables(cls, metadata):
         Table('parent', metadata,
-            Column('id', Integer, primary_key=True),
+            Column('pid', Integer, primary_key=True),
             Column('pdata', String(30))
         )
         Table('child', metadata,
-            Column('id', Integer, primary_key=True),
-            Column('pid', Integer, ForeignKey('parent.id')),
+            Column('cid', Integer, primary_key=True),
+            Column('pid', Integer, ForeignKey('parent.pid')),
             Column('cdata', String(30))
         )
 
@@ -2367,27 +2373,27 @@ class InheritingRowSwitchTest(fixtures.MappedTest):
         mapper(C, child, inherits=P)
 
         sess = create_session()
-        c1 = C(id=1, pdata='c1', cdata='c1')
+        c1 = C(pid=1, cid=1, pdata='c1', cdata='c1')
         sess.add(c1)
         sess.flush()
 
         # establish a row switch between c1 and c2.
         # c2 has no value for the "child" table
-        c2 = C(id=1, pdata='c2')
+        c2 = C(pid=1, cid=1, pdata='c2')
         sess.add(c2)
         sess.delete(c1)
 
         self.assert_sql_execution(testing.db, sess.flush,
-            CompiledSQL("UPDATE parent SET pdata=:pdata WHERE parent.id = :parent_id",
-                {'pdata':'c2', 'parent_id':1}
+            CompiledSQL("UPDATE parent SET pdata=:pdata WHERE parent.pid = :parent_pid",
+                {'pdata':'c2', 'parent_pid':1}
             ),
 
             # this fires as of [ticket:1362], since we synchronzize
             # PK/FKs on UPDATES.  c2 is new so the history shows up as
             # pure added, update occurs.  If a future change limits the
             # sync operation during _save_obj().update, this is safe to remove again.
-            CompiledSQL("UPDATE child SET pid=:pid WHERE child.id = :child_id",
-                {'pid':1, 'child_id':1}
+            CompiledSQL("UPDATE child SET pid=:pid WHERE child.cid = :child_cid",
+                {'pid':1, 'child_cid':1}
             )
         )
 
@@ -2525,4 +2531,4 @@ class PartialNullPKTest(fixtures.MappedTest):
         s.commit()
 
         t.col1 = "1"
-        s.commit()
+        s.commit()
\ No newline at end of file
