@@ -13,10 +13,12 @@ from sqlalchemy.testing.exclusions import \
      only_if,\
      only_on,\
      fails_on_everything_except,\
+     fails_on,\
      fails_if,\
      succeeds_if,\
      SpecPredicate,\
-     against
+     against,\
+     LambdaPredicate
 
 def no_support(db, reason):
     return SpecPredicate(db, description=reason)
@@ -61,6 +63,13 @@ class DefaultRequirements(SuiteRequirements):
                     'target backend does not support ON UPDATE CASCADE'
                 )
 
+    @property
+    def non_updating_cascade(self):
+        """target database must *not* support ON UPDATE..CASCADE behavior in
+        foreign keys."""
+
+        return fails_on_everything_except('sqlite', 'oracle', '+zxjdbc') + skip_if('mssql')
+
     @property
     def deferrable_fks(self):
         """target database must support deferrable fks"""
@@ -85,8 +94,6 @@ class DefaultRequirements(SuiteRequirements):
             no_support('oracle', 'not supported by database'),
             no_support('mssql', 'not supported by database'),
             no_support('sybase', 'not supported by database'),
-            no_support('maxdb', 'FIXME: verify not supported by database'),
-            no_support('informix', 'not supported by database'),
         ])
 
     @property
@@ -124,6 +131,18 @@ class DefaultRequirements(SuiteRequirements):
                 "not supported by database"
             )
 
+    @property
+    def insert_from_select(self):
+        return skip_if(
+                    ["firebird"], "crashes for unknown reason"
+                )
+
+    @property
+    def fetch_rows_post_commit(self):
+        return skip_if(
+                    ["firebird"], "not supported"
+                )
+
     @property
     def binary_comparisons(self):
         """target database/driver can allow BLOB/BINARY fields to be compared
@@ -133,6 +152,26 @@ class DefaultRequirements(SuiteRequirements):
                 "not supported by database/driver"
             )
 
+    @property
+    def binary_literals(self):
+        """target backend supports simple binary literals, e.g. an
+        expression like::
+
+            SELECT CAST('foo' AS BINARY)
+
+        Where ``BINARY`` is the type emitted from :class:`.LargeBinary`,
+        e.g. it could be ``BLOB`` or similar.
+
+        Basically fails on Oracle.
+
+        """
+        # adding mssql here since it doesn't support comparisons either,
+        # have observed generally bad behavior with binary / mssql.
+
+        return skip_if(["oracle", "mssql"],
+                "not supported by database/driver"
+            )
+
     @property
     def independent_cursors(self):
         """Target must support simultaneous, independent database cursors
@@ -226,7 +265,6 @@ class DefaultRequirements(SuiteRequirements):
                     "sqlite",
                     "sybase",
                     ("mysql", "<", (5, 0, 3)),
-                    ("informix", "<", (11, 55, "xC3"))
                     ], "savepoints not supported")
 
 
@@ -283,14 +321,14 @@ class DefaultRequirements(SuiteRequirements):
         """Target database must support INTERSECT or equivalent."""
 
         return fails_if([
-                "firebird", "mysql", "sybase", "informix"
+                "firebird", "mysql", "sybase",
             ], 'no support for INTERSECT')
 
     @property
     def except_(self):
         """Target database must support EXCEPT or equivalent (i.e. MINUS)."""
         return fails_if([
-                "firebird", "mysql", "sybase", "informix"
+                "firebird", "mysql", "sybase",
             ], 'no support for EXCEPT')
 
     @property
@@ -313,7 +351,6 @@ class DefaultRequirements(SuiteRequirements):
 
         return skip_if([
             no_support('firebird', 'no SA implementation'),
-            no_support('maxdb', 'two-phase xact not supported by database'),
             no_support('mssql', 'two-phase xact not supported by drivers'),
             no_support('oracle', 'two-phase xact not implemented in SQLA/oracle'),
             no_support('drizzle', 'two-phase xact not supported by database'),
@@ -324,6 +361,7 @@ class DefaultRequirements(SuiteRequirements):
                        'need separate XA implementation'),
             exclude('mysql', '<', (5, 0, 3),
                         'two-phase xact not supported by database'),
+            no_support("postgresql+pg8000", "not supported and/or hangs")
             ])
 
     @property
@@ -365,36 +403,36 @@ class DefaultRequirements(SuiteRequirements):
     def unicode_ddl(self):
         """Target driver must support some degree of non-ascii symbol names."""
         # TODO: expand to exclude MySQLdb versions w/ broken unicode
+
         return skip_if([
-            no_support('maxdb', 'database support flakey'),
             no_support('oracle', 'FIXME: no support in database?'),
             no_support('sybase', 'FIXME: guessing, needs confirmation'),
             no_support('mssql+pymssql', 'no FreeTDS support'),
+            LambdaPredicate(
+                lambda config: against(config, 'mssql+pyodbc') and
+                config.db.dialect.freetds and
+                config.db.dialect.freetds_driver_version < "0.91",
+                "older freetds doesn't support unicode DDL"
+            ),
             exclude('mysql', '<', (4, 1, 1), 'no unicode connection support'),
-            ])
+        ])
 
     @property
     def sane_rowcount(self):
         return skip_if(
-            lambda: not self.db.dialect.supports_sane_rowcount,
+            lambda config: not config.db.dialect.supports_sane_rowcount,
             "driver doesn't support 'sane' rowcount"
         )
 
-    @property
-    def cextensions(self):
-        return skip_if(
-                lambda: not self._has_cextensions(), "C extensions not installed"
-                )
 
     @property
     def emulated_lastrowid(self):
         """"target dialect retrieves cursor.lastrowid or an equivalent
         after an insert() construct executes.
         """
-        return fails_on_everything_except('mysql+mysqldb', 'mysql+oursql',
-                                      'sqlite+pysqlite', 'mysql+pymysql',
-                                      'mysql+cymysql',
-                                      'sybase', 'mssql+pyodbc', 'mssql+mxodbc')
+        return fails_on_everything_except('mysql',
+                                      'sqlite+pysqlite',
+                                      'sybase', 'mssql')
 
     @property
     def implements_get_lastrowid(self):
@@ -408,14 +446,14 @@ class DefaultRequirements(SuiteRequirements):
         cursor object.
 
         """
-        return fails_on_everything_except('mysql+mysqldb', 'mysql+oursql',
-                                       'sqlite+pysqlite', 'mysql+pymysql',
-                                       'mysql+cymysql')
+        return skip_if('mssql+pymssql', 'crashes on pymssql') + \
+                    fails_on_everything_except('mysql',
+                                       'sqlite+pysqlite')
 
     @property
     def sane_multi_rowcount(self):
-        return skip_if(
-                    lambda: not self.db.dialect.supports_sane_multi_rowcount,
+        return fails_if(
+                    lambda config: not config.db.dialect.supports_sane_multi_rowcount,
                     "driver doesn't support 'sane' multi row count"
                 )
 
@@ -431,6 +469,15 @@ class DefaultRequirements(SuiteRequirements):
         return fails_on_everything_except('postgresql', 'oracle', 'mssql',
                     'sybase')
 
+    @property
+    def datetime_literals(self):
+        """target dialect supports rendering of a date, time, or datetime as a
+        literal string, e.g. via the TypeEngine.literal_processor() method.
+
+        """
+
+        return fails_on_everything_except("sqlite")
+
     @property
     def datetime(self):
         """target dialect supports representation of Python
@@ -460,6 +507,13 @@ class DefaultRequirements(SuiteRequirements):
 
         return exclusions.open()
 
+    @property
+    def date_coerces_from_datetime(self):
+        """target dialect accepts a datetime object as the target
+        of a date column."""
+
+        return fails_on('mysql+mysqlconnector')
+
     @property
     def date_historic(self):
         """target dialect supports representation of Python
@@ -482,27 +536,29 @@ class DefaultRequirements(SuiteRequirements):
         return skip_if(['mssql', 'mysql', 'firebird', '+zxjdbc',
                     'oracle', 'sybase'])
 
+
     @property
     def precision_numerics_general(self):
         """target backend has general support for moderately high-precision
         numerics."""
-        return fails_if('mssql+pymssql', 'FIXME: improve pymssql dec handling')
+        return exclusions.open()
 
     @property
     def precision_numerics_enotation_small(self):
         """target backend supports Decimal() objects using E notation
         to represent very small values."""
-        return fails_if('mssql+pymssql', 'FIXME: improve pymssql dec handling')
+        # NOTE: this exclusion isn't used in current tests.
+        return exclusions.open()
 
     @property
     def precision_numerics_enotation_large(self):
         """target backend supports Decimal() objects using E notation
         to represent very large values."""
 
-        return fails_if(
-                ("sybase+pyodbc", None, None,
+        return skip_if(
+                [("sybase+pyodbc", None, None,
                     "Don't know how do get these values through FreeTDS + Sybase"),
-                ("firebird", None, None, "Precision must be from 1 to 18"),
+                ("firebird", None, None, "Precision must be from 1 to 18"),]
             )
 
     @property
@@ -529,13 +585,48 @@ class DefaultRequirements(SuiteRequirements):
                     ('oracle', None, None,
             "this may be a bug due to the difficulty in handling "
             "oracle precision numerics"),
-                    ('postgresql+pg8000', None, None,
-        "pg-8000 does native decimal but truncates the decimals."),
                     ("firebird", None, None,
         "database and/or driver truncates decimal places.")
                 ]
                 )
 
+    @property
+    def precision_generic_float_type(self):
+        """target backend will return native floating point numbers with at
+        least seven decimal places when using the generic Float type."""
+
+        return fails_if([
+                    ('mysql', None, None,
+                                'mysql FLOAT type only returns 4 decimals'),
+                    ('firebird', None, None,
+                                "firebird FLOAT type isn't high precision"),
+                ])
+
+    @property
+    def floats_to_four_decimals(self):
+        return fails_if([
+                    ("mysql+oursql", None, None, "Floating point error"),
+                    ("firebird", None, None,
+                        "Firebird still has FP inaccuracy even "
+                        "with only four decimal places"),
+                    ('mssql+pyodbc', None, None,
+                                'mssql+pyodbc has FP inaccuracy even with '
+                                'only four decimal places '
+                            ),
+                    ('mssql+pymssql', None, None,
+                                'mssql+pymssql has FP inaccuracy even with '
+                                'only four decimal places '),
+                    (
+                        'postgresql+pg8000', None, None,
+                        'postgresql+pg8000 has FP inaccuracy even with '
+                        'only four decimal places '),
+                ])
+
+    @property
+    def fetch_null_from_numeric(self):
+        return skip_if(
+                    ("mssql+pyodbc", None, None, "crashes due to bug #351"),
+                )
 
     @property
     def python2(self):
@@ -551,35 +642,23 @@ class DefaultRequirements(SuiteRequirements):
                 "Python version 3.xx is required."
                 )
 
-    @property
-    def python26(self):
-        return skip_if(
-                lambda: sys.version_info < (2, 6),
-                "Python version 2.6 or greater is required"
-            )
-
-    @property
-    def python25(self):
-        return skip_if(
-                lambda: sys.version_info < (2, 5),
-                "Python version 2.5 or greater is required"
-            )
-
     @property
     def cpython(self):
         return only_if(lambda: util.cpython,
                "cPython interpreter needed"
              )
 
+
     @property
-    def bulletproof_pickle(self):
+    def non_broken_pickle(self):
         from sqlalchemy.util import pickle
         return only_if(
-            lambda: pickle.__name__ == 'cPickle' and \
-                sys.version_info < (3, 0) and not util.pypy,
-            "Needs Python 2.x cPickle"
+            lambda: not util.pypy and pickle.__name__ == 'cPickle'
+                or sys.version_info >= (3, 2),
+            "Needs cPickle+cPython or newer Python 3 pickle"
         )
 
+
     @property
     def predictable_gc(self):
         """target platform must remove all cycles unconditionally when
@@ -590,11 +669,11 @@ class DefaultRequirements(SuiteRequirements):
 
     @property
     def hstore(self):
-        def check_hstore():
-            if not against("postgresql"):
+        def check_hstore(config):
+            if not against(config, "postgresql"):
                 return False
             try:
-                self.db.execute("SELECT 'a=>1,a=>2'::hstore;")
+                config.db.execute("SELECT 'a=>1,a=>2'::hstore;")
                 return True
             except:
                 return False
@@ -603,11 +682,11 @@ class DefaultRequirements(SuiteRequirements):
 
     @property
     def range_types(self):
-        def check_range_types():
-            if not against("postgresql+psycopg2"):
+        def check_range_types(config):
+            if not against(config, "postgresql+psycopg2"):
                 return False
             try:
-                self.db.execute("select '[1,2)'::int4range;")
+                config.db.execute("select '[1,2)'::int4range;")
                 # only supported in psycopg 2.5+
                 from psycopg2.extras import NumericRange
                 return True
@@ -616,28 +695,34 @@ class DefaultRequirements(SuiteRequirements):
 
         return only_if(check_range_types)
 
-    @property
-    def sqlite(self):
-        return skip_if(lambda: not self._has_sqlite())
 
     @property
     def oracle_test_dblink(self):
         return skip_if(
-                    lambda: not self.config.file_config.has_option(
+                    lambda config: not config.file_config.has_option(
                         'sqla_testing', 'oracle_db_link'),
                     "oracle_db_link option not specified in config"
                 )
 
     @property
-    def ad_hoc_engines(self):
-        """Test environment must allow ad-hoc engine/connection creation.
+    def percent_schema_names(self):
+        return skip_if(
+                [
+                    ("+psycopg2", None, None,
+                            "psycopg2 2.4 no longer accepts % in bind placeholders"),
+                    ("mysql", None, None, "executemany() doesn't work here")
+                ]
+            )
 
-        DBs that scale poorly for many connections, even when closed, i.e.
-        Oracle, may use the "--low-connections" option which flags this requirement
-        as not present.
+    @property
+    def order_by_label_with_expression(self):
+        return fails_if([
+                    ('firebird', None, None, "kinterbasdb doesn't send full type information"),
+                    ('postgresql', None, None, 'only simple labels allowed'),
+                    ('sybase', None, None, 'only simple labels allowed'),
+                    ('mssql', None, None, 'only simple labels allowed')
+                ])
 
-        """
-        return skip_if(lambda: self.config.options.low_connections)
 
     @property
     def skip_mysql_on_windows(self):
@@ -654,41 +739,26 @@ class DefaultRequirements(SuiteRequirements):
 
         """
         return skip_if(
-                lambda: (util.py3k or not util.py26) and
-                    self.config.options.enable_plugin_coverage,
-                "Stability issues with coverage + py3k, py2.5"
+                lambda config: util.py3k and
+                    config.options.has_coverage,
+                "Stability issues with coverage + py3k"
             )
 
-    @property
-    def english_locale_on_postgresql(self):
-        return skip_if(lambda: against('postgresql') \
-                    and not self.db.scalar('SHOW LC_COLLATE').startswith('en'))
-
     @property
     def selectone(self):
         """target driver must support the literal statement 'select 1'"""
         return skip_if(["oracle", "firebird"], "non-standard SELECT scalar syntax")
 
-    def _has_cextensions(self):
-        try:
-            from sqlalchemy import cresultproxy, cprocessors
-            return True
-        except ImportError:
-            return False
 
-    def _has_sqlite(self):
-        from sqlalchemy import create_engine
-        try:
-            create_engine('sqlite://')
-            return True
-        except ImportError:
-            return False
+    @property
+    def mysql_fully_case_sensitive(self):
+        return only_if(self._has_mysql_fully_case_sensitive)
 
-    def _has_mysql_on_windows(self):
-        return against('mysql') and \
-                self.db.dialect._detect_casing(self.db) == 1
+    def _has_mysql_on_windows(self, config):
+        return against(config, 'mysql') and \
+                config.db.dialect._detect_casing(config.db) == 1
 
-    def _has_mysql_fully_case_sensitive(self):
-        return against('mysql') and \
-                self.db.dialect._detect_casing(self.db) == 0
+    def _has_mysql_fully_case_sensitive(self, config):
+        return against(config, 'mysql') and \
+                config.db.dialect._detect_casing(config.db) == 0
 
